import functools
import os
from tspi_exceptions import *

from cffi import FFI, VerificationError
INTERFACE_H = os.path.dirname(os.path.abspath(__file__)) + '/interface.h'
__all__ = ["ffi", "lib"]

# Setup CFFI with libtspi
ffi = FFI()

ffi.cdef(open(INTERFACE_H, 'r').read())
tss_lib = ffi.verify('#include <trousers/tss.h>', libraries=['tspi'])

def wrap_libtspi_func(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        ret = func(*args, **kwargs)
        if ret == 0:
            return True
        if (ret & 0x3000):
            ret = ret & ~0x3000
            if ret == tss_lib.TSS_E_FAIL:
                raise TSS_E_FAIL
            elif ret == tss_lib.TSS_E_BAD_PARAMETER:
                raise TSS_E_BAD_PARAMETER
            elif ret == tss_lib.TSS_E_INTERNAL_ERROR:
                raise TSS_E_INTERNAL_ERROR
            elif ret == tss_lib.TSS_E_OUTOFMEMORY:
                raise TSS_E_OUTOFMEMORY
            elif ret == tss_lib.TSS_E_NOTIMPL:
                raise TSS_E_NOTIMPL
            elif ret == tss_lib.TSS_E_KEY_ALREADY_REGISTERED:
                raise TSS_E_KEY_ALREADY_REGISTERED
            elif ret == tss_lib.TSS_E_TPM_UNEXPECTED:
                raise TSS_E_TPM_UNEXPECTED
            elif ret == tss_lib.TSS_E_COMM_FAILURE:
                raise TSS_E_COMM_FAILURE
            elif ret == tss_lib.TSS_E_TIMEOUT:
                raise TSS_E_TIMEOUT
            elif ret == tss_lib.TSS_E_TPM_UNSUPPORTED_FEATURE:
                raise TSS_E_TPM_UNSUPPORTED_FEATURE
            elif ret == tss_lib.TSS_E_CANCELED:
                raise TSS_E_CANCELED
            elif ret == tss_lib.TSS_E_PS_KEY_NOTFOUND:
                raise TSS_E_PS_KEY_NOTFOUND
            elif ret == tss_lib.TSS_E_PS_KEY_EXISTS:
                raise TSS_E_PS_KEY_EXISTS
            elif ret == tss_lib.TSS_E_PS_BAD_KEY_STATE:
                raise TSS_E_PS_BAD_KEY_STATE
            elif ret == tss_lib.TSS_E_INVALID_OBJECT_TYPE:
                raise TSS_E_INVALID_OBJECT_TYPE
            elif ret == tss_lib.TSS_E_NO_CONNECTION:
                raise TSS_E_NO_CONNECTION
            elif ret == tss_lib.TSS_E_CONNECTION_FAILED:
                raise TSS_E_CONNECTION_FAILED
            elif ret == tss_lib.TSS_E_CONNECTION_BROKEN:
                raise TSS_E_CONNECTION_BROKEN
            elif ret == tss_lib.TSS_E_HASH_INVALID_ALG:
                raise TSS_E_HASH_INVALID_ALG
            elif ret == tss_lib.TSS_E_HASH_INVALID_LENGTH:
                raise TSS_E_HASH_INVALID_LENGTH
            elif ret == tss_lib.TSS_E_HASH_NO_DATA:
                raise TSS_E_HASH_NO_DATA
            elif ret == tss_lib.TSS_E_INVALID_ATTRIB_FLAG:
                raise TSS_E_INVALID_ATTRIB_FLAG
            elif ret == tss_lib.TSS_E_INVALID_ATTRIB_SUBFLAG:
                raise TSS_E_INVALID_ATTRIB_SUBFLAG
            elif ret == tss_lib.TSS_E_INVALID_ATTRIB_DATA:
                raise TSS_E_INVALID_ATTRIB_DATA
            elif ret == tss_lib.TSS_E_INVALID_OBJECT_INITFLAG:
                raise TSS_E_INVALID_OBJECT_INITFLAG
            elif ret == tss_lib.TSS_E_NO_PCRS_SET:
                raise TSS_E_NO_PCRS_SET
            elif ret == tss_lib.TSS_E_KEY_NOT_LOADED:
                raise TSS_E_KEY_NOT_LOADED
            elif ret == tss_lib.TSS_E_KEY_NOT_SET:
                raise TSS_E_KEY_NOT_SET
            elif ret == tss_lib.TSS_E_VALIDATION_FAILED:
                raise TSS_E_VALIDATION_FAILED
            elif ret == tss_lib.TSS_E_TSP_AUTHREQUIRED:
                raise TSS_E_TSP_AUTHREQUIRED
            elif ret == tss_lib.TSS_E_TSP_AUTH2REQUIRED:
                raise TSS_E_TSP_AUTH2REQUIRED
            elif ret == tss_lib.TSS_E_TSP_AUTHFAIL:
                raise TSS_E_TSP_AUTHFAIL
            elif ret == tss_lib.TSS_E_TSP_AUTH2FAIL:
                raise TSS_E_TSP_AUTH2FAIL
            elif ret == tss_lib.TSS_E_KEY_NO_MIGRATION_POLICY:
                raise TSS_E_KEY_NO_MIGRATION_POLICY
            elif ret == tss_lib.TSS_E_POLICY_NO_SECRET:
                raise TSS_E_POLICY_NO_SECRET
            elif ret == tss_lib.TSS_E_INVALID_OBJ_ACCESS:
                raise TSS_E_INVALID_OBJ_ACCESS
            elif ret == tss_lib.TSS_E_INVALID_ENCSCHEME:
                raise TSS_E_INVALID_ENCSCHEME
            elif ret == tss_lib.TSS_E_INVALID_SIGSCHEME:
                raise TSS_E_INVALID_SIGSCHEME
            elif ret == tss_lib.TSS_E_ENC_INVALID_LENGTH:
                raise TSS_E_ENC_INVALID_LENGTH
            elif ret == tss_lib.TSS_E_ENC_NO_DATA:
                raise TSS_E_ENC_NO_DATA
            elif ret == tss_lib.TSS_E_ENC_INVALID_TYPE:
                raise TSS_E_ENC_INVALID_TYPE
            elif ret == tss_lib.TSS_E_INVALID_KEYUSAGE:
                raise TSS_E_INVALID_KEYUSAGE
            elif ret == tss_lib.TSS_E_VERIFICATION_FAILED:
                raise TSS_E_VERIFICATION_FAILED
            elif ret == tss_lib.TSS_E_HASH_NO_IDENTIFIER:
                raise TSS_E_HASH_NO_IDENTIFIER
            elif ret == tss_lib.TSS_E_INVALID_HANDLE:
                raise TSS_E_INVALID_HANDLE
            elif ret == tss_lib.TSS_E_SILENT_CONTEXT:
                raise TSS_E_SILENT_CONTEXT
            elif ret == tss_lib.TSS_E_EK_CHECKSUM:
                raise TSS_E_EK_CHECKSUM
            elif ret == tss_lib.TSS_E_DELEGATION_NOTSET:
                raise TSS_E_DELEGATION_NOTSET
            elif ret == tss_lib.TSS_E_DELFAMILY_NOTFOUND:
                raise TSS_E_DELFAMILY_NOTFOUND
            elif ret == tss_lib.TSS_E_DELFAMILY_ROWEXISTS:
                raise TSS_E_DELFAMILY_ROWEXISTS
            elif ret == tss_lib.TSS_E_VERSION_MISMATCH:
                raise TSS_E_VERSION_MISMATCH
            elif ret == tss_lib.TSS_E_DAA_AR_DECRYPTION_ERROR:
                raise TSS_E_DAA_AR_DECRYPTION_ERROR
            elif ret == tss_lib.TSS_E_DAA_AUTHENTICATION_ERROR:
                raise TSS_E_DAA_AUTHENTICATION_ERROR
            elif ret == tss_lib.TSS_E_DAA_CHALLENGE_RESPONSE_ERROR:
                raise TSS_E_DAA_CHALLENGE_RESPONSE_ERROR
            elif ret == tss_lib.TSS_E_DAA_CREDENTIAL_PROOF_ERROR:
                raise TSS_E_DAA_CREDENTIAL_PROOF_ERROR
            elif ret == tss_lib.TSS_E_DAA_CREDENTIAL_REQUEST_PROOF_ERROR:
                raise TSS_E_DAA_CREDENTIAL_REQUEST_PROOF_ERROR
            elif ret == tss_lib.TSS_E_DAA_ISSUER_KEY_ERROR:
                raise TSS_E_DAA_ISSUER_KEY_ERROR
            elif ret == tss_lib.TSS_E_DAA_PSEUDONYM_ERROR:
                raise TSS_E_DAA_PSEUDONYM_ERROR
            elif ret == tss_lib.TSS_E_INVALID_RESOURCE:
                raise TSS_E_INVALID_RESOURCE
            elif ret == tss_lib.TSS_E_NV_AREA_EXIST:
                raise TSS_E_NV_AREA_EXIST
            elif ret == tss_lib.TSS_E_NV_AREA_NOT_EXIST:
                raise TSS_E_NV_AREA_NOT_EXIST
            elif ret == tss_lib.TSS_E_TSP_TRANS_AUTHFAIL:
                raise TSS_E_TSP_TRANS_AUTHFAIL
            elif ret == tss_lib.TSS_E_TSP_TRANS_AUTHREQUIRED:
                raise TSS_E_TSP_TRANS_AUTHREQUIRED
            elif ret == tss_lib.TSS_E_TSP_TRANS_NOTEXCLUSIVE:
                raise TSS_E_TSP_TRANS_NOTEXCLUSIVE
            elif ret == tss_lib.TSS_E_TSP_TRANS_FAIL:
                raise TSS_E_TSP_TRANS_FAIL
            elif ret == tss_lib.TSS_E_TSP_TRANS_NO_PUBKEY:
                raise TSS_E_TSP_TRANS_NO_PUBKEY
            elif ret == tss_lib.TSS_E_NO_ACTIVE_COUNTER:
                raise TSS_E_NO_ACTIVE_COUNTER
            else:
                raise TspiException("Unknown Error %x" % ret)
        else:
            if ret == tss_lib.TPM_E_NON_FATAL:
                raise TPM_E_NON_FATAL
            elif ret == tss_lib.TPM_E_AUTHFAIL:
                raise TPM_E_AUTHFAIL
            elif ret == tss_lib.TPM_E_BADINDEX:
                raise TPM_E_BADINDEX
            elif ret == tss_lib.TPM_E_BAD_PARAMETER:
                raise TPM_E_BAD_PARAMETER
            elif ret == tss_lib.TPM_E_AUDITFAILURE:
                raise TPM_E_AUDITFAILURE
            elif ret == tss_lib.TPM_E_CLEAR_DISABLED:
                raise TPM_E_CLEAR_DISABLED
            elif ret == tss_lib.TPM_E_DEACTIVATED:
                raise TPM_E_DEACTIVATED
            elif ret == tss_lib.TPM_E_DISABLED:
                raise TPM_E_DISABLED
            elif ret == tss_lib.TPM_E_DISABLED_CMD:
                raise TPM_E_DISABLED_CMD
            elif ret == tss_lib.TPM_E_FAIL:
                raise TPM_E_FAIL
            elif ret == tss_lib.TPM_E_BAD_ORDINAL:
                raise TPM_E_BAD_ORDINAL
            elif ret == tss_lib.TPM_E_INSTALL_DISABLED:
                raise TPM_E_INSTALL_DISABLED
            elif ret == tss_lib.TPM_E_INVALID_KEYHANDLE:
                raise TPM_E_INVALID_KEYHANDLE
            elif ret == tss_lib.TPM_E_KEYNOTFOUND:
                raise TPM_E_KEYNOTFOUND
            elif ret == tss_lib.TPM_E_INAPPROPRIATE_ENC:
                raise TPM_E_INAPPROPRIATE_ENC
            elif ret == tss_lib.TPM_E_MIGRATEFAIL:
                raise TPM_E_MIGRATEFAIL
            elif ret == tss_lib.TPM_E_INVALID_PCR_INFO:
                raise TPM_E_INVALID_PCR_INFO
            elif ret == tss_lib.TPM_E_NOSPACE:
                raise TPM_E_NOSPACE
            elif ret == tss_lib.TPM_E_NOSRK:
                raise TPM_E_NOSRK
            elif ret == tss_lib.TPM_E_NOTSEALED_BLOB:
                raise TPM_E_NOTSEALED_BLOB
            elif ret == tss_lib.TPM_E_OWNER_SET:
                raise TPM_E_OWNER_SET
            elif ret == tss_lib.TPM_E_RESOURCES:
                raise TPM_E_RESOURCES
            elif ret == tss_lib.TPM_E_SHORTRANDOM:
                raise TPM_E_SHORTRANDOM
            elif ret == tss_lib.TPM_E_SIZE:
                raise TPM_E_SIZE
            elif ret == tss_lib.TPM_E_WRONGPCRVAL:
                raise TPM_E_WRONGPCRVAL
            elif ret == tss_lib.TPM_E_BAD_PARAM_SIZE:
                raise TPM_E_BAD_PARAM_SIZE
            elif ret == tss_lib.TPM_E_SHA_THREAD:
                raise TPM_E_SHA_THREAD
            elif ret == tss_lib.TPM_E_SHA_ERROR:
                raise TPM_E_SHA_ERROR
            elif ret == tss_lib.TPM_E_FAILEDSELFTEST:
                raise TPM_E_FAILEDSELFTEST
            elif ret == tss_lib.TPM_E_AUTH2FAIL:
                raise TPM_E_AUTH2FAIL
            elif ret == tss_lib.TPM_E_BADTAG:
                raise TPM_E_BADTAG
            elif ret == tss_lib.TPM_E_IOERROR:
                raise TPM_E_IOERROR
            elif ret == tss_lib.TPM_E_ENCRYPT_ERROR:
                raise TPM_E_ENCRYPT_ERROR
            elif ret == tss_lib.TPM_E_DECRYPT_ERROR:
                raise TPM_E_DECRYPT_ERROR
            elif ret == tss_lib.TPM_E_INVALID_AUTHHANDLE:
                raise TPM_E_INVALID_AUTHHANDLE
            elif ret == tss_lib.TPM_E_NO_ENDORSEMENT:
                raise TPM_E_NO_ENDORSEMENT
            elif ret == tss_lib.TPM_E_INVALID_KEYUSAGE:
                raise TPM_E_INVALID_KEYUSAGE
            elif ret == tss_lib.TPM_E_WRONG_ENTITYTYPE:
                raise TPM_E_WRONG_ENTITYTYPE
            elif ret == tss_lib.TPM_E_INVALID_POSTINIT:
                raise TPM_E_INVALID_POSTINIT
            elif ret == tss_lib.TPM_E_INAPPROPRIATE_SIG:
                raise TPM_E_INAPPROPRIATE_SIG
            elif ret == tss_lib.TPM_E_BAD_KEY_PROPERTY:
                raise TPM_E_BAD_KEY_PROPERTY
            elif ret == tss_lib.TPM_E_BAD_MIGRATION:
                raise TPM_E_BAD_MIGRATION
            elif ret == tss_lib.TPM_E_BAD_SCHEME:
                raise TPM_E_BAD_SCHEME
            elif ret == tss_lib.TPM_E_BAD_DATASIZE:
                raise TPM_E_BAD_DATASIZE
            elif ret == tss_lib.TPM_E_BAD_MODE:
                raise TPM_E_BAD_MODE
            elif ret == tss_lib.TPM_E_BAD_PRESENCE:
                raise TPM_E_BAD_PRESENCE
            elif ret == tss_lib.TPM_E_BAD_VERSION:
                raise TPM_E_BAD_VERSION
            elif ret == tss_lib.TPM_E_NO_WRAP_TRANSPORT:
                raise TPM_E_NO_WRAP_TRANSPORT
            elif ret == tss_lib.TPM_E_AUDITFAIL_UNSUCCESSFUL:
                raise TPM_E_AUDITFAIL_UNSUCCESSFUL
            elif ret == tss_lib.TPM_E_AUDITFAIL_SUCCESSFUL:
                raise TPM_E_AUDITFAIL_SUCCESSFUL
            elif ret == tss_lib.TPM_E_NOTRESETABLE:
                raise TPM_E_NOTRESETABLE
            elif ret == tss_lib.TPM_E_NOTLOCAL:
                raise TPM_E_NOTLOCAL
            elif ret == tss_lib.TPM_E_BAD_TYPE:
                raise TPM_E_BAD_TYPE
            elif ret == tss_lib.TPM_E_INVALID_RESOURCE:
                raise TPM_E_INVALID_RESOURCE
            elif ret == tss_lib.TPM_E_NOTFIPS:
                raise TPM_E_NOTFIPS
            elif ret == tss_lib.TPM_E_INVALID_FAMILY:
                raise TPM_E_INVALID_FAMILY
            elif ret == tss_lib.TPM_E_NO_NV_PERMISSION:
                raise TPM_E_NO_NV_PERMISSION
            elif ret == tss_lib.TPM_E_REQUIRES_SIGN:
                raise TPM_E_REQUIRES_SIGN
            elif ret == tss_lib.TPM_E_KEY_NOTSUPPORTED:
                raise TPM_E_KEY_NOTSUPPORTED
            elif ret == tss_lib.TPM_E_AUTH_CONFLICT:
                raise TPM_E_AUTH_CONFLICT
            elif ret == tss_lib.TPM_E_AREA_LOCKED:
                raise TPM_E_AREA_LOCKED
            elif ret == tss_lib.TPM_E_BAD_LOCALITY:
                raise TPM_E_BAD_LOCALITY
            elif ret == tss_lib.TPM_E_READ_ONLY:
                raise TPM_E_READ_ONLY
            elif ret == tss_lib.TPM_E_PER_NOWRITE:
                raise TPM_E_PER_NOWRITE
            elif ret == tss_lib.TPM_E_FAMILYCOUNT:
                raise TPM_E_FAMILYCOUNT
            elif ret == tss_lib.TPM_E_WRITE_LOCKED:
                raise TPM_E_WRITE_LOCKED
            elif ret == tss_lib.TPM_E_BAD_ATTRIBUTES:
                raise TPM_E_BAD_ATTRIBUTES
            elif ret == tss_lib.TPM_E_INVALID_STRUCTURE:
                raise TPM_E_INVALID_STRUCTURE
            elif ret == tss_lib.TPM_E_KEY_OWNER_CONTROL:
                raise TPM_E_KEY_OWNER_CONTROL
            elif ret == tss_lib.TPM_E_BAD_COUNTER:
                raise TPM_E_BAD_COUNTER
            elif ret == tss_lib.TPM_E_NOT_FULLWRITE:
                raise TPM_E_NOT_FULLWRITE
            elif ret == tss_lib.TPM_E_CONTEXT_GAP:
                raise TPM_E_CONTEXT_GAP
            elif ret == tss_lib.TPM_E_MAXNVWRITES:
                raise TPM_E_MAXNVWRITES
            elif ret == tss_lib.TPM_E_NOOPERATOR:
                raise TPM_E_NOOPERATOR
            elif ret == tss_lib.TPM_E_RESOURCEMISSING:
                raise TPM_E_RESOURCEMISSING
            elif ret == tss_lib.TPM_E_DELEGATE_LOCK:
                raise TPM_E_DELEGATE_LOCK
            elif ret == tss_lib.TPM_E_DELEGATE_FAMILY:
                raise TPM_E_DELEGATE_FAMILY
            elif ret == tss_lib.TPM_E_DELEGATE_ADMIN:
                raise TPM_E_DELEGATE_ADMIN
            elif ret == tss_lib.TPM_E_TRANSPORT_NOTEXCLUSIVE:
                raise TPM_E_TRANSPORT_NOTEXCLUSIVE
            elif ret == tss_lib.TPM_E_OWNER_CONTROL:
                raise TPM_E_OWNER_CONTROL
            elif ret == tss_lib.TPM_E_DAA_RESOURCES:
                raise TPM_E_DAA_RESOURCES
            elif ret == tss_lib.TPM_E_DAA_INPUT_DATA0:
                raise TPM_E_DAA_INPUT_DATA0
            elif ret == tss_lib.TPM_E_DAA_INPUT_DATA1:
                raise TPM_E_DAA_INPUT_DATA1
            elif ret == tss_lib.TPM_E_DAA_ISSUER_SETTINGS:
                raise TPM_E_DAA_ISSUER_SETTINGS
            elif ret == tss_lib.TPM_E_DAA_TPM_SETTINGS:
                raise TPM_E_DAA_TPM_SETTINGS
            elif ret == tss_lib.TPM_E_DAA_STAGE:
                raise TPM_E_DAA_STAGE
            elif ret == tss_lib.TPM_E_DAA_ISSUER_VALIDITY:
                raise TPM_E_DAA_ISSUER_VALIDITY
            elif ret == tss_lib.TPM_E_DAA_WRONG_W:
                raise TPM_E_DAA_WRONG_W
            elif ret == tss_lib.TPM_E_BAD_HANDLE:
                raise TPM_E_BAD_HANDLE
            elif ret == tss_lib.TPM_E_BAD_DELEGATE:
                raise TPM_E_BAD_DELEGATE
            elif ret == tss_lib.TPM_E_BADCONTEXT:
                raise TPM_E_BADCONTEXT
            elif ret == tss_lib.TPM_E_TOOMANYCONTEXTS:
                raise TPM_E_TOOMANYCONTEXTS
            elif ret == tss_lib.TPM_E_MA_TICKET_SIGNATURE:
                raise TPM_E_MA_TICKET_SIGNATURE
            elif ret == tss_lib.TPM_E_MA_DESTINATION:
                raise TPM_E_MA_DESTINATION
            elif ret == tss_lib.TPM_E_MA_SOURCE:
                raise TPM_E_MA_SOURCE
            elif ret == tss_lib.TPM_E_MA_AUTHORITY:
                raise TPM_E_MA_AUTHORITY
            elif ret == tss_lib.TPM_E_PERMANENTEK:
                raise TPM_E_PERMANENTEK
            elif ret == tss_lib.TPM_E_BAD_SIGNATURE:
                raise TPM_E_BAD_SIGNATURE
            elif ret == tss_lib.TPM_E_NOCONTEXTSPACE:
                raise TPM_E_NOCONTEXTSPACE
            elif ret == tss_lib.TPM_E_RETRY:
                raise TPM_E_RETRY
            elif ret == tss_lib.TPM_E_NEEDS_SELFTEST:
                raise TPM_E_NEEDS_SELFTEST
            elif ret == tss_lib.TPM_E_DOING_SELFTEST:
                raise TPM_E_DOING_SELFTEST
            elif ret == tss_lib.TPM_E_DEFEND_LOCK_RUNNING:
                raise TPM_E_DEFEND_LOCK_RUNNING
            else:
                raise TpmException("Unknown Error %x" % ret)
            
    return wrapper

tss_lib.Tspi_EncodeDER_TssBlob = wrap_libtspi_func(tss_lib.Tspi_EncodeDER_TssBlob)
tss_lib.Tspi_DecodeBER_TssBlob = wrap_libtspi_func(tss_lib.Tspi_DecodeBER_TssBlob)
tss_lib.Tspi_SetAttribUint32 = wrap_libtspi_func(tss_lib.Tspi_SetAttribUint32)
tss_lib.Tspi_GetAttribUint32 = wrap_libtspi_func(tss_lib.Tspi_GetAttribUint32)
tss_lib.Tspi_SetAttribData = wrap_libtspi_func(tss_lib.Tspi_SetAttribData)
tss_lib.Tspi_GetAttribData = wrap_libtspi_func(tss_lib.Tspi_GetAttribData)
tss_lib.Tspi_ChangeAuth = wrap_libtspi_func(tss_lib.Tspi_ChangeAuth)
tss_lib.Tspi_ChangeAuthAsym = wrap_libtspi_func(tss_lib.Tspi_ChangeAuthAsym)
tss_lib.Tspi_GetPolicyObject = wrap_libtspi_func(tss_lib.Tspi_GetPolicyObject)
tss_lib.Tspi_Context_Create = wrap_libtspi_func(tss_lib.Tspi_Context_Create)
tss_lib.Tspi_Context_Close = wrap_libtspi_func(tss_lib.Tspi_Context_Close)
tss_lib.Tspi_Context_Connect = wrap_libtspi_func(tss_lib.Tspi_Context_Connect)
tss_lib.Tspi_Context_FreeMemory = wrap_libtspi_func(tss_lib.Tspi_Context_FreeMemory)
tss_lib.Tspi_Context_GetDefaultPolicy = wrap_libtspi_func(tss_lib.Tspi_Context_GetDefaultPolicy)
tss_lib.Tspi_Context_CreateObject = wrap_libtspi_func(tss_lib.Tspi_Context_CreateObject)
tss_lib.Tspi_Context_CloseObject = wrap_libtspi_func(tss_lib.Tspi_Context_CloseObject)
tss_lib.Tspi_Context_GetCapability = wrap_libtspi_func(tss_lib.Tspi_Context_GetCapability)
tss_lib.Tspi_Context_GetTpmObject = wrap_libtspi_func(tss_lib.Tspi_Context_GetTpmObject)
tss_lib.Tspi_Context_SetTransEncryptionKey = wrap_libtspi_func(tss_lib.Tspi_Context_SetTransEncryptionKey)
tss_lib.Tspi_Context_CloseSignTransport = wrap_libtspi_func(tss_lib.Tspi_Context_CloseSignTransport)
tss_lib.Tspi_Context_LoadKeyByBlob = wrap_libtspi_func(tss_lib.Tspi_Context_LoadKeyByBlob)
tss_lib.Tspi_Context_LoadKeyByUUID = wrap_libtspi_func(tss_lib.Tspi_Context_LoadKeyByUUID)
tss_lib.Tspi_Context_RegisterKey = wrap_libtspi_func(tss_lib.Tspi_Context_RegisterKey)
tss_lib.Tspi_Context_UnregisterKey = wrap_libtspi_func(tss_lib.Tspi_Context_UnregisterKey)
tss_lib.Tspi_Context_GetKeyByUUID = wrap_libtspi_func(tss_lib.Tspi_Context_GetKeyByUUID)
tss_lib.Tspi_Context_GetKeyByPublicInfo = wrap_libtspi_func(tss_lib.Tspi_Context_GetKeyByPublicInfo)
tss_lib.Tspi_Context_GetRegisteredKeysByUUID = wrap_libtspi_func(tss_lib.Tspi_Context_GetRegisteredKeysByUUID)
tss_lib.Tspi_Context_GetRegisteredKeysByUUID2 = wrap_libtspi_func(tss_lib.Tspi_Context_GetRegisteredKeysByUUID2)
tss_lib.Tspi_Policy_SetSecret = wrap_libtspi_func(tss_lib.Tspi_Policy_SetSecret)
tss_lib.Tspi_Policy_FlushSecret = wrap_libtspi_func(tss_lib.Tspi_Policy_FlushSecret)
tss_lib.Tspi_Policy_AssignToObject = wrap_libtspi_func(tss_lib.Tspi_Policy_AssignToObject)
tss_lib.Tspi_TPM_KeyControlOwner = wrap_libtspi_func(tss_lib.Tspi_TPM_KeyControlOwner)
tss_lib.Tspi_TPM_CreateEndorsementKey = wrap_libtspi_func(tss_lib.Tspi_TPM_CreateEndorsementKey)
tss_lib.Tspi_TPM_CreateRevocableEndorsementKey = wrap_libtspi_func(tss_lib.Tspi_TPM_CreateRevocableEndorsementKey)
tss_lib.Tspi_TPM_RevokeEndorsementKey = wrap_libtspi_func(tss_lib.Tspi_TPM_RevokeEndorsementKey)
tss_lib.Tspi_TPM_GetPubEndorsementKey = wrap_libtspi_func(tss_lib.Tspi_TPM_GetPubEndorsementKey)
tss_lib.Tspi_TPM_OwnerGetSRKPubKey = wrap_libtspi_func(tss_lib.Tspi_TPM_OwnerGetSRKPubKey)
tss_lib.Tspi_TPM_TakeOwnership = wrap_libtspi_func(tss_lib.Tspi_TPM_TakeOwnership)
tss_lib.Tspi_TPM_ClearOwner = wrap_libtspi_func(tss_lib.Tspi_TPM_ClearOwner)
tss_lib.Tspi_TPM_CollateIdentityRequest = wrap_libtspi_func(tss_lib.Tspi_TPM_CollateIdentityRequest)
tss_lib.Tspi_TPM_ActivateIdentity = wrap_libtspi_func(tss_lib.Tspi_TPM_ActivateIdentity)
tss_lib.Tspi_TPM_CreateMaintenanceArchive = wrap_libtspi_func(tss_lib.Tspi_TPM_CreateMaintenanceArchive)
tss_lib.Tspi_TPM_KillMaintenanceFeature = wrap_libtspi_func(tss_lib.Tspi_TPM_KillMaintenanceFeature)
tss_lib.Tspi_TPM_LoadMaintenancePubKey = wrap_libtspi_func(tss_lib.Tspi_TPM_LoadMaintenancePubKey)
tss_lib.Tspi_TPM_CheckMaintenancePubKey = wrap_libtspi_func(tss_lib.Tspi_TPM_CheckMaintenancePubKey)
tss_lib.Tspi_TPM_SetOperatorAuth = wrap_libtspi_func(tss_lib.Tspi_TPM_SetOperatorAuth)
tss_lib.Tspi_TPM_SetStatus = wrap_libtspi_func(tss_lib.Tspi_TPM_SetStatus)
tss_lib.Tspi_TPM_GetStatus = wrap_libtspi_func(tss_lib.Tspi_TPM_GetStatus)
tss_lib.Tspi_TPM_GetCapability = wrap_libtspi_func(tss_lib.Tspi_TPM_GetCapability)
tss_lib.Tspi_TPM_GetCapabilitySigned = wrap_libtspi_func(tss_lib.Tspi_TPM_GetCapabilitySigned)
tss_lib.Tspi_TPM_SelfTestFull = wrap_libtspi_func(tss_lib.Tspi_TPM_SelfTestFull)
tss_lib.Tspi_TPM_CertifySelfTest = wrap_libtspi_func(tss_lib.Tspi_TPM_CertifySelfTest)
tss_lib.Tspi_TPM_GetTestResult = wrap_libtspi_func(tss_lib.Tspi_TPM_GetTestResult)
tss_lib.Tspi_TPM_GetRandom = wrap_libtspi_func(tss_lib.Tspi_TPM_GetRandom)
tss_lib.Tspi_TPM_StirRandom = wrap_libtspi_func(tss_lib.Tspi_TPM_StirRandom)
tss_lib.Tspi_TPM_GetEvent = wrap_libtspi_func(tss_lib.Tspi_TPM_GetEvent)
tss_lib.Tspi_TPM_GetEvents = wrap_libtspi_func(tss_lib.Tspi_TPM_GetEvents)
tss_lib.Tspi_TPM_GetEventLog = wrap_libtspi_func(tss_lib.Tspi_TPM_GetEventLog)
tss_lib.Tspi_TPM_Quote = wrap_libtspi_func(tss_lib.Tspi_TPM_Quote)
tss_lib.Tspi_TPM_Quote2 = wrap_libtspi_func(tss_lib.Tspi_TPM_Quote2)
tss_lib.Tspi_TPM_PcrExtend = wrap_libtspi_func(tss_lib.Tspi_TPM_PcrExtend)
tss_lib.Tspi_TPM_PcrRead = wrap_libtspi_func(tss_lib.Tspi_TPM_PcrRead)
tss_lib.Tspi_TPM_PcrReset = wrap_libtspi_func(tss_lib.Tspi_TPM_PcrReset)
tss_lib.Tspi_TPM_AuthorizeMigrationTicket = wrap_libtspi_func(tss_lib.Tspi_TPM_AuthorizeMigrationTicket)
tss_lib.Tspi_TPM_CMKSetRestrictions = wrap_libtspi_func(tss_lib.Tspi_TPM_CMKSetRestrictions)
tss_lib.Tspi_TPM_CMKApproveMA = wrap_libtspi_func(tss_lib.Tspi_TPM_CMKApproveMA)
tss_lib.Tspi_TPM_CMKCreateTicket = wrap_libtspi_func(tss_lib.Tspi_TPM_CMKCreateTicket)
tss_lib.Tspi_TPM_ReadCounter = wrap_libtspi_func(tss_lib.Tspi_TPM_ReadCounter)
tss_lib.Tspi_TPM_ReadCurrentTicks = wrap_libtspi_func(tss_lib.Tspi_TPM_ReadCurrentTicks)
tss_lib.Tspi_TPM_DirWrite = wrap_libtspi_func(tss_lib.Tspi_TPM_DirWrite)
tss_lib.Tspi_TPM_DirRead = wrap_libtspi_func(tss_lib.Tspi_TPM_DirRead)
tss_lib.Tspi_TPM_Delegate_AddFamily = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_AddFamily)
tss_lib.Tspi_TPM_Delegate_GetFamily = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_GetFamily)
tss_lib.Tspi_TPM_Delegate_InvalidateFamily = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_InvalidateFamily)
tss_lib.Tspi_TPM_Delegate_CreateDelegation = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_CreateDelegation)
tss_lib.Tspi_TPM_Delegate_CacheOwnerDelegation = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_CacheOwnerDelegation)
tss_lib.Tspi_TPM_Delegate_UpdateVerificationCount = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_UpdateVerificationCount)
tss_lib.Tspi_TPM_Delegate_VerifyDelegation = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_VerifyDelegation)
tss_lib.Tspi_TPM_Delegate_ReadTables = wrap_libtspi_func(tss_lib.Tspi_TPM_Delegate_ReadTables)
tss_lib.Tspi_TPM_GetAuditDigest = wrap_libtspi_func(tss_lib.Tspi_TPM_GetAuditDigest)
tss_lib.Tspi_PcrComposite_SelectPcrIndex = wrap_libtspi_func(tss_lib.Tspi_PcrComposite_SelectPcrIndex)
tss_lib.Tspi_PcrComposite_SelectPcrIndexEx = wrap_libtspi_func(tss_lib.Tspi_PcrComposite_SelectPcrIndexEx)
tss_lib.Tspi_PcrComposite_SetPcrValue = wrap_libtspi_func(tss_lib.Tspi_PcrComposite_SetPcrValue)
tss_lib.Tspi_PcrComposite_GetPcrValue = wrap_libtspi_func(tss_lib.Tspi_PcrComposite_GetPcrValue)
tss_lib.Tspi_PcrComposite_SetPcrLocality = wrap_libtspi_func(tss_lib.Tspi_PcrComposite_SetPcrLocality)
tss_lib.Tspi_PcrComposite_GetPcrLocality = wrap_libtspi_func(tss_lib.Tspi_PcrComposite_GetPcrLocality)
tss_lib.Tspi_PcrComposite_GetCompositeHash = wrap_libtspi_func(tss_lib.Tspi_PcrComposite_GetCompositeHash)
tss_lib.Tspi_Key_LoadKey = wrap_libtspi_func(tss_lib.Tspi_Key_LoadKey)
tss_lib.Tspi_Key_UnloadKey = wrap_libtspi_func(tss_lib.Tspi_Key_UnloadKey)
tss_lib.Tspi_Key_GetPubKey = wrap_libtspi_func(tss_lib.Tspi_Key_GetPubKey)
tss_lib.Tspi_Key_CertifyKey = wrap_libtspi_func(tss_lib.Tspi_Key_CertifyKey)
tss_lib.Tspi_Key_CreateKey = wrap_libtspi_func(tss_lib.Tspi_Key_CreateKey)
tss_lib.Tspi_Key_WrapKey = wrap_libtspi_func(tss_lib.Tspi_Key_WrapKey)
tss_lib.Tspi_Key_CreateMigrationBlob = wrap_libtspi_func(tss_lib.Tspi_Key_CreateMigrationBlob)
tss_lib.Tspi_Key_ConvertMigrationBlob = wrap_libtspi_func(tss_lib.Tspi_Key_ConvertMigrationBlob)
tss_lib.Tspi_Key_CMKCreateBlob = wrap_libtspi_func(tss_lib.Tspi_Key_CMKCreateBlob)
tss_lib.Tspi_Key_CMKConvertMigration = wrap_libtspi_func(tss_lib.Tspi_Key_CMKConvertMigration)
tss_lib.Tspi_Hash_Sign = wrap_libtspi_func(tss_lib.Tspi_Hash_Sign)
tss_lib.Tspi_Hash_VerifySignature = wrap_libtspi_func(tss_lib.Tspi_Hash_VerifySignature)
tss_lib.Tspi_Hash_SetHashValue = wrap_libtspi_func(tss_lib.Tspi_Hash_SetHashValue)
tss_lib.Tspi_Hash_GetHashValue = wrap_libtspi_func(tss_lib.Tspi_Hash_GetHashValue)
tss_lib.Tspi_Hash_UpdateHashValue = wrap_libtspi_func(tss_lib.Tspi_Hash_UpdateHashValue)
tss_lib.Tspi_Hash_TickStampBlob = wrap_libtspi_func(tss_lib.Tspi_Hash_TickStampBlob)
tss_lib.Tspi_Data_Bind = wrap_libtspi_func(tss_lib.Tspi_Data_Bind)
tss_lib.Tspi_Data_Unbind = wrap_libtspi_func(tss_lib.Tspi_Data_Unbind)
tss_lib.Tspi_Data_Seal = wrap_libtspi_func(tss_lib.Tspi_Data_Seal)
tss_lib.Tspi_Data_Unseal = wrap_libtspi_func(tss_lib.Tspi_Data_Unseal)
tss_lib.Tspi_NV_DefineSpace = wrap_libtspi_func(tss_lib.Tspi_NV_DefineSpace)
tss_lib.Tspi_NV_ReleaseSpace = wrap_libtspi_func(tss_lib.Tspi_NV_ReleaseSpace)
tss_lib.Tspi_NV_WriteValue = wrap_libtspi_func(tss_lib.Tspi_NV_WriteValue)
tss_lib.Tspi_NV_ReadValue = wrap_libtspi_func(tss_lib.Tspi_NV_ReadValue)



from interface import tss_lib, ffi
import tspi_exceptions
import hashlib


def uuid_to_tss_uuid(uuid):
    """Converts a Python UUID into a TSS UUID"""
    tss_uuid = ffi.new('struct tdTSS_UUID *')[0]
    tss_uuid.ulTimeLow = uuid.time_low
    tss_uuid.usTimeMid = uuid.time_mid
    tss_uuid.usTimeHigh = uuid.time_hi_version
    tss_uuid.bClockSeqHigh = uuid.clock_seq_hi_variant
    tss_uuid.bClockSeqLow = uuid.clock_seq_low
    tss_uuid.rgbNode[0] = (uuid.node >> 40) & 0xff
    tss_uuid.rgbNode[1] = (uuid.node >> 32) & 0xff
    tss_uuid.rgbNode[2] = (uuid.node >> 24) & 0xff
    tss_uuid.rgbNode[3] = (uuid.node >> 16) & 0xff
    tss_uuid.rgbNode[4] = (uuid.node >> 8) & 0xff
    tss_uuid.rgbNode[5] = uuid.node & 0xff

    return tss_uuid


class TspiObject(object):
    def __init__(self, context, ctype, tss_type, flags, handle=None):
        """
        Init a TSPI object
        
        :param context: The TSS context to use
        :param ctype: The C type associated with this TSS object
        :param tss_type: The TSS type associated with this TSS object
        :param flags: The default attributes of the object
        :param handle: Use an existing handle, rather than creating a new
        object
        """
        self.context = context
        if handle is not None:
            self.handle = handle
        else:
            self.handle = ffi.new(ctype)
            tss_lib.Tspi_Context_CreateObject(context, tss_type, flags,
                                          self.handle)

    def get_handle(self):
        """Return the TSS handle for the object"""        
        return self.handle[0]

    def set_attribute_uint32(self, attrib, sub, val):
        """
        Set a 32 bit attribute associated with a given object
        
        :param attrib: The attribute to modify
        :param sub: The subattribute to modify
        :param val: The value to assign
        """
        tss_lib.Tspi_SetAttribUint32(self.get_handle(), attrib, sub, val)

    def set_attribute_data(self, attrib, sub, data):
        """
        Set an arbitrary datatype attribute associated with the object

        :param attrib: The attribute to modify
        :param sub: The subattribute to modify
        :param val: The data to assign
        """
        cdata = ffi.new('BYTE[]', len(data))
        for i in range(len(data)):
            cdata[i] = data[i]
        tss_lib.Tspi_SetAttribData(self.get_handle(), attrib, sub, len(data), cdata)

    def get_attribute_data(self, attrib, sub):
        """
        Get an arbitrary datatype associated with the object

        :param attrib: The attribute to modify
        :param sub: The subattribute to modify

        :returns: a bytearray containing the data
        """
        bloblen = ffi.new('UINT32 *')
        blob = ffi.new('BYTE **')
        tss_lib.Tspi_GetAttribData(self.handle[0], attrib, sub, bloblen, blob)
        ret = bytearray(blob[0][0:bloblen[0]])
        tss_lib.Tspi_Context_FreeMemory(self.context, blob[0])
        return ret

    def get_policy_object(self, poltype):
        """
        Get a policy object assigned to the object

        :param poltype: The policy object type

        :returns: A TspiPolicy
        """
        policy = ffi.new('TSS_HPOLICY *')
        tss_lib.Tspi_GetPolicyObject(self.get_handle(), poltype, policy)
        policy_obj = TspiPolicy(self.context, None, handle=policy)
        return policy_obj

    def generate_random_data(self, length):
        """
        Generates random data in TPM of the given length in bytes.

        :param length: Length of data to generate in bytes

        :return: bytearray containing the random data
        """
        blob = ffi.new('BYTE **')
        tss_lib.Tspi_TPM_GetRandom(self.get_handle(), length, blob)
        ret = bytearray(blob[0][0:length])
        tss_lib.Tspi_Context_FreeMemory(self.context, blob[0])
        return ret


class TspiNV(TspiObject):
    def __init__(self, context, flags):
        super(TspiNV, self).__init__(context, 'TSS_HNVSTORE *',
                                     tss_lib.TSS_OBJECT_TYPE_NV, flags)

    def read_value(self, offset, length):
        """
        Read a value from TPM NVRAM

        :param offset: The offset in NVRAM to start reading
        :param length: The number of bytes of NVRAM to read
        :returns: A bytearray containing the requested data
        """
        lenval = ffi.new('UINT32 *')
        data = ffi.new('BYTE **')
        lenval[0] = length
        tss_lib.Tspi_NV_ReadValue(self.handle[0], offset, lenval, data)
        ret = bytearray(data[0][0:lenval[0]])
        return ret

    def set_index(self, index):
        """
        Select the requested NVRAM storage area index

        :param index: The storage area index to select
        """
        tss_lib.Tspi_SetAttribUint32(self.handle[0], tss_lib.TSS_TSPATTRIB_NV_INDEX,
                                 0, index)


class TspiPolicy(TspiObject):
    def __init__(self, context, flags, handle=None):
        super(TspiPolicy, self).__init__(context, 'TSS_HPOLICY *',
                                         tss_lib.TSS_OBJECT_TYPE_POLICY, flags,
                                         handle)

    def set_secret(self, sectype, secret):
        """
        Set the authorisation data of a policy object

        :param sectype: The type of the secret, any of the constants
            prefixed TSS_SECRET_MODE_ in tspi_defines
        :param secret: The secret data blob as either a string or
            array of integers in the range 0..255
        """
        tss_lib.Tspi_Policy_SetSecret(self.handle[0], sectype, len(secret),
                                      _c_byte_array(secret))

    def assign(self, target):
        """
        Assign a policy to an object

        :param target: The object to which the policy will be assigned
        """
        tss_lib.Tspi_Policy_AssignToObject(self.handle[0], target.get_handle())


class TspiPCRs(TspiObject):
    def __init__(self, context, flags):
        self.pcrs = {}
        super(TspiPCRs, self).__init__(context, 'TSS_HPCRS *',
                                       tss_lib.TSS_OBJECT_TYPE_PCRS, flags)

    def set_pcrs(self, pcrs):
        """
        Set the PCRs referred to by this object

        :param pcrs: A list of integer PCRs
        """
        for pcr in pcrs:
            tss_lib.Tspi_PcrComposite_SelectPcrIndex(self.handle[0], pcr)
            self.pcrs[pcr] = ""

    def get_pcrs(self):
        """
        Get the digest value of the PCRs referred to by this object

        :returns: a dictionary of PCR/value pairs
        """
        for pcr in self.pcrs:
            buf = ffi.new('BYTE **')
            buflen = ffi.new('UINT32 *')
            tss_lib.Tspi_PcrComposite_GetPcrValue(self.handle[0], pcr, buflen, buf)
            self.pcrs[pcr] = bytearray(buf[0][0:buflen[0]])
            tss_lib.Tspi_Context_FreeMemory(self.context, buf[0])
        return self.pcrs

class TspiHash(TspiObject):
    def __init__(self, context, flags):
        super(TspiHash, self).__init__(context, 'TSS_HHASH *',
                                       tss_lib.TSS_OBJECT_TYPE_HASH, flags)

    def update(self, data):
        """
        Update the hash object with new data

        :param data: The data to hash
        """
        tss_lib.Tspi_Hash_UpdateHashValue(self.get_handle(), len(data),
                                          _c_byte_array(data))

    def verify(self, key, signature):
        """
        Verify that the hash matches a given signature

        :param key: A TspiObject representing the key to use
        :param signature: The signature to compare against
        """
        tss_lib.Tspi_Hash_VerifySignature(self.get_handle(), key.get_handle(),
                                     len(signature), _c_byte_array(signature))

    def sign(self, key):
        """
        Sign this hash with the specified key and return a signature

        :param key: a TspiKey instance corresponding to a loaded key
        :return: a string of bytes containing the signature
        """
        csig_size = ffi.new("UINT32*")
        csig_data = ffi.new("BYTE**")
        tss_lib.Tspi_Hash_Sign(self.get_handle(), key.get_handle(), csig_size, csig_data)
        return ffi.buffer(csig_data[0], csig_size[0])


class TspiKey(TspiObject):
    def __init__(self, context, flags, handle=None):
        self.context = context
        super(TspiKey, self).__init__(context, 'TSS_HKEY *',
                                      tss_lib.TSS_OBJECT_TYPE_RSAKEY,
                                      flags, handle)

    def __del__(self):
        try:
            tss_lib.Tspi_Key_UnloadKey(self.get_handle())
        # The key may have been implicitly unloaded as part of a previous
        # operation
        except tspi_exceptions.TSS_E_INVALID_HANDLE:
            pass

    def set_modulus(self, n):
        """
        Set the key modulus

        :param n: The key modulus
        """
        self.set_attribute_data(tss_lib.TSS_TSPATTRIB_RSAKEY_INFO,
                                tss_lib.TSS_TSPATTRIB_KEYINFO_RSA_MODULUS, n)

    def get_keyblob(self):
        """
        Obtain a TSS blob corresponding to the key

        :returns: a bytearray containing the TSS key blob
        """
        return self.get_attribute_data(tss_lib.TSS_TSPATTRIB_KEY_BLOB,
                                       tss_lib.TSS_TSPATTRIB_KEYBLOB_BLOB)

    def get_pubkeyblob(self):
        """
        Obtain a TSS blob corresponding to the public portion of the key

        :returns: a bytearray containing the TSS key blob
        """
        return self.get_attribute_data(tss_lib.TSS_TSPATTRIB_KEY_BLOB,
                                       tss_lib.TSS_TSPATTRIB_KEYBLOB_PUBLIC_KEY)

    def get_pubkey(self):
        """
        Obtain the public part of the key

        :returns: a bytearray containing the public portion of the key
        """
        return self.get_attribute_data(tss_lib.TSS_TSPATTRIB_RSAKEY_INFO,
                                       tss_lib.TSS_TSPATTRIB_KEYINFO_RSA_MODULUS)


    def seal(self, data, pcrs=None):
        """
        Seal data to the local TPM using this key

        :param data: The data to seal
        :param pcrs: A list of PCRs to seal the data to

        :returns: a bytearray of the encrypted data
        """
        encdata = TspiObject(self.context, 'TSS_HENCDATA *',
                             tss_lib.TSS_OBJECT_TYPE_ENCDATA,
                             tss_lib.TSS_ENCDATA_SEAL)

        if pcrs is not None:
            pcrobj=TspiPCRs(self.context, tss_lib.TSS_PCRS_STRUCT_INFO)
            pcrobj.set_pcrs(pcrs)
            pcr_composite = pcrobj.get_handle()
        else:
            pcr_composite = 0

        cdata = ffi.new('BYTE[]', len(data))
        for i in range(len(data)):
            cdata[i] = data[i]

        tss_lib.Tspi_Data_Seal(encdata.get_handle(), self.get_handle(),
                               len(data), cdata, pcr_composite)
        blob = encdata.get_attribute_data(tss_lib.TSS_TSPATTRIB_ENCDATA_BLOB,
                                    tss_lib.TSS_TSPATTRIB_ENCDATABLOB_BLOB)
        return bytearray(blob)

    def unseal(self, data):
        """
        Unseal data from the local TPM using this key

        :param data: The data to unseal

        :returns: a bytearray of the unencrypted data
        """
        encdata = TspiObject(self.context, 'TSS_HENCDATA *',
                             tss_lib.TSS_OBJECT_TYPE_ENCDATA,
                             tss_lib.TSS_ENCDATA_SEAL)

        encdata.set_attribute_data(tss_lib.TSS_TSPATTRIB_ENCDATA_BLOB,
                                tss_lib.TSS_TSPATTRIB_ENCDATABLOB_BLOB, data)

        bloblen = ffi.new('UINT32 *')
        blob = ffi.new('BYTE **')

        tss_lib.Tspi_Data_Unseal(encdata.get_handle(), self.get_handle(),
                                 bloblen, blob)
        ret = bytearray(blob[0][0:bloblen[0]])
        tss_lib.Tspi_Context_FreeMemory(self.context, blob[0])
        return ret


class TspiTPM(TspiObject):
    def __init__(self, context):
        tpm = ffi.new('TSS_HTPM *')
        tss_lib.Tspi_Context_GetTpmObject(context, tpm)
        self.handle = tpm
        self.context = context

    def collate_identity_request(self, srk, pubkey, aik):
        """
        Generate everything required to authenticate the TPM to a third party

        :param srk: The storage root key to use
        :param pubkey: The key to use for signing the output key
        :param aik: The key to use as the identity key

        :returns: A bytearray containing a certificate request
        """
        bloblen = ffi.new('UINT32 *')
        blob = ffi.new('BYTE **')
        tss_lib.Tspi_TPM_CollateIdentityRequest(self.get_handle(), 
                                            srk.get_handle(),
                                            pubkey.get_handle(), 0, "",
                                            aik.get_handle(), tss_lib.TSS_ALG_AES,
                                            bloblen, blob)
        ret = bytearray(blob[0][0:bloblen[0]])
        tss_lib.Tspi_Context_FreeMemory(self.context, blob[0])
        return ret

    def get_capability(self, cap, sub):
        """
        Get information on the capabilities of the TPM

        :param cap: The capability to query
        :param sub: The subcapability to query

        :returns: A bytearray containing the capability data
        """
        resp = ffi.new('BYTE **')
        resplen = ffi.new('UINT32 *')
        csub = ffi.new('BYTE []', len(sub))
        for i in range(len(sub)):
            csub[i] = sub[i]
        tss_lib.Tspi_TPM_Getcapability(self.handle[0], cap, len(sub), csub,
                                   resplen, resp)
        ret = bytearray(resp[0][0:resplen[0]])
        tss_lib.Tspi_Context_FreeMemory(self.context, resp[0])
        return ret

    def get_quote(self, aik, pcrs, challenge):
        """
        retrieve a signed set of PCR values

        :param aik: A TspiObject representing the Attestation Identity Key
        :param pcrs: A TspiPCRs representing the PCRs to be quoted
        :param challenge: The challenge to use

        :returns: A tuple containing the quote data and the validation block
        """
        valid = ffi.new('TSS_VALIDATION *')
        chalmd = ffi.new('BYTE[]', 20)

        if challenge:
            m = hashlib.sha1()
            m.update(challenge)
            sha1 = bytearray(m.digest())
            for i in range(len(sha1)):
                chalmd[i] = sha1[i]

        valid[0].ulExternalDataLength = ffi.sizeof(chalmd)
        valid[0].rgbExternalData = chalmd

        tss_lib.Tspi_TPM_Quote(self.handle[0], aik.get_handle(), pcrs.get_handle(),
                           valid)

        data = bytearray(valid[0].rgbData[0:valid[0].ulDataLength])
        validation = bytearray(valid[0].rgbValidationData
                               [0:valid[0].ulValidationDataLength])
        tss_lib.Tspi_Context_FreeMemory(self.context, valid[0].rgbData)
        tss_lib.Tspi_Context_FreeMemory(self.context, valid[0].rgbValidationData)
        return (data, validation)

    def activate_identity(self, aik, asymblob, symblob):
        """
        Decrypt the challenge provided by the attestation host

        :param aik: A TspiObject representing the Attestation Identity Key
        :param asymblob: The asymmetrically encrypted challenge data
        :param symblob: The symmetrically encrypted challenge data

        :returns: A bytearray containing the decrypted challenge
        """
        casymblob = ffi.new('BYTE[]', len(asymblob))
        for i in range(len(asymblob)):
            casymblob[i] = asymblob[i]
        csymblob = ffi.new('BYTE[]', len(symblob))
        for i in range(len(symblob)):
            csymblob[i] = symblob[i]
        credlen = ffi.new('UINT32 *')
        cred = ffi.new('BYTE **')
        tss_lib.Tspi_TPM_ActivateIdentity(self.handle[0], aik.get_handle(),
                                      len(asymblob), casymblob,
                                      len(symblob), csymblob, credlen, cred)
        ret = bytearray(cred[0][0:credlen[0]])
        tss_lib.Tspi_Context_FreeMemory(self.context, cred[0])
        return ret

    def get_pub_endorsement_key(self):
        keyblob = ffi.new('TSS_HKEY *')
        modulus = ffi.new('UINT32 *')
        modlen = ffi.new('BYTE **')
        tss_lib.Tspi_TPM_GetPubEndorsementKey(self.get_handle(), 1, ffi.NULL,
                                              keyblob)
        key = TspiKey(self.context, None, handle=keyblob)
        return key

    def take_ownership(self, srk):
        """
        Take ownership of the TPM

        :param srk: The Storage Root Key to use
        """
        tss_lib.Tspi_TPM_TakeOwnership(self.get_handle(), srk.get_handle(), 0)

    def extend_pcr(self, pcr, data, event):
        """
        Extend a PCR

        :param pcr: The PCR to extend
        :param data: The data to be hashed by the TPM for extending the PCR
        :param event: A dict containing the event data

        :returns: A bytearray containing the new PCR value
        """
        cdata = ffi.new('BYTE []', len(data))
        bloblen = ffi.new('UINT32 *')
        blob = ffi.new('BYTE **')
        for i in range(len(data)):
            cdata[i] = data[i]

        tss_lib.Tspi_TPM_PcrExtend(self.get_handle(), pcr, len(data), cdata,
                                   ffi.NULL, bloblen, blob)
        ret = bytearray(blob[0][0:bloblen[0]])
        tss_lib.Tspi_Context_FreeMemory(self.context, blob[0])
        return ret

class TspiContext():
    def __init__(self):
        self.context = ffi.new('TSS_HCONTEXT *')
        tss_lib.Tspi_Context_Create(self.context)
        self.context = self.context[0]
        self.tpm = None

    def __del__(self):
        tss_lib.Tspi_Context_Close(self.context)

    def connect(self, host=None):
        """
        Connect a context to a TSS daemon

        :param host: The host to connect to, if not localhost
        """
        if host is not None:
            chost = ffi.new('uint16_t[]', len(host) + 1)
            for i in range(len(host)):
                chost[i] = bytearray(host)[i]
            chost[len(host)] = 0
            tss_lib.Tspi_Context_Connect(self.context, chost)
        else:
            tss_lib.Tspi_Context_Connect(self.context, ffi.NULL)
        self.tpm = TspiTPM(self.context)

    def create_nv(self, flags):
        """
        Create a TspiNV object associated with this context

        :param flags: Flags to pass
        """
        obj = TspiNV(self.context, flags)
        return obj

    def create_policy(self, flags):
        """
        Create a TspiPolicy object associated with this context

        :param flags: Flags to pass
        """
        obj = TspiPolicy(self.context, flags)
        return obj

    def create_pcrs(self, flags):
        """
        Create a TspiPCRs object associated with this context

        :param flags: Flags to pass
        """
        obj = TspiPCRs(self.context, flags)
        return obj

    def create_hash(self, flags):
        """
        Create a TspiHash object associated with this context

        :param flags: Flags to pass
        """
        obj = TspiHash(self.context, flags)
        return obj

    def create_rsa_key(self, flags):
        """
        Create a Tspi key object associated with this context

        :param flags: Flags to pass
        """
        obj = TspiKey(self.context, flags)
        return obj

    def load_key_by_uuid(self, storagetype, uuid):
        """
        Load a key that's been registered in persistent storage

        :param storagetype: The key storage type
        :param uuid: The UUID associated with the key

        :returns: a TspiKey
        """
        tss_key = ffi.new('TSS_HKEY *')
        tss_uuid = uuid_to_tss_uuid(uuid)
        tss_lib.Tspi_Context_LoadKeyByUUID(self.context, storagetype, tss_uuid,
                                       tss_key)
        key = TspiKey(self.context, None, handle=tss_key)
        return key

    def load_key_by_blob(self, srk, blob):
        """
        Load a key from a TSS key blob

        :param srk: A TspiObject representing the Storage Root key
        :param blob: The TSS key blob

        :returns: A TspiKey
        """
        tss_key = ffi.new('TSS_HKEY *')
        cblob = ffi.new('BYTE[]', len(blob))
        for i in range(len(blob)):
            cblob[i] = blob[i]
        tss_lib.Tspi_Context_LoadKeyByBlob(self.context, srk.get_handle(),
                                       len(blob), cblob, tss_key)
        key = TspiKey(self.context, None, handle=tss_key)
        return key

    def get_tpm_object(self):
        """Returns the TspiTPM associated with this context"""        
        return self.tpm


def _c_byte_array(data):
    """
    Creates and returns a ffi BYTE[] type containing data.
    :param data: a string of bytes or array of integers in range 0x00..0xff
    :return: ffi cdata instance backed by a c BYTE[] structure containing
        the contents of data
    """
    cdata = ffi.new('BYTE []', len(data))
    if isinstance(data, basestring):
        data = bytearray(data)
    for i in range(len(data)):
        cdata[i] = data[i]
    return cdata



from interface import tss_lib

TSS_NV_DEFINED = tss_lib.TSS_NV_DEFINED
TPM_NV_INDEX_LOCK = tss_lib.TPM_NV_INDEX_LOCK
TPM_NV_INDEX0 = tss_lib.TPM_NV_INDEX0
TPM_NV_INDEX_DIR = tss_lib.TPM_NV_INDEX_DIR
TPM_NV_INDEX_EKCert = tss_lib.TPM_NV_INDEX_EKCert
TPM_NV_INDEX_TPM_CC = tss_lib.TPM_NV_INDEX_TPM_CC
TPM_NV_INDEX_PlatformCert = tss_lib.TPM_NV_INDEX_PlatformCert
TPM_NV_INDEX_Platform_CC = tss_lib.TPM_NV_INDEX_Platform_CC
TPM_NV_INDEX_TSS_BASE = tss_lib.TPM_NV_INDEX_TSS_BASE
TPM_NV_INDEX_PC_BASE = tss_lib.TPM_NV_INDEX_PC_BASE
TPM_NV_INDEX_SERVER_BASE = tss_lib.TPM_NV_INDEX_SERVER_BASE
TPM_NV_INDEX_MOBILE_BASE = tss_lib.TPM_NV_INDEX_MOBILE_BASE
TPM_NV_INDEX_PERIPHERAL_BASE = tss_lib.TPM_NV_INDEX_PERIPHERAL_BASE
TPM_NV_INDEX_GROUP_RESV_BASE = tss_lib.TPM_NV_INDEX_GROUP_RESV_BASE

TSS_POLICY_USAGE = tss_lib.TSS_POLICY_USAGE
TSS_POLICY_MIGRATION = tss_lib.TSS_POLICY_MIGRATION
TSS_POLICY_OPERATOR = tss_lib.TSS_POLICY_OPERATOR

TSS_SECRET_LIFETIME_ALWAYS = tss_lib.TSS_SECRET_LIFETIME_ALWAYS
TSS_SECRET_LIFETIME_COUNTER = tss_lib.TSS_SECRET_LIFETIME_COUNTER
TSS_SECRET_LIFETIME_TIMER = tss_lib.TSS_SECRET_LIFETIME_TIMER
TSS_SECRET_MODE_NONE = tss_lib.TSS_SECRET_MODE_NONE
TSS_SECRET_MODE_SHA1 = tss_lib.TSS_SECRET_MODE_SHA1
TSS_SECRET_MODE_PLAIN = tss_lib.TSS_SECRET_MODE_PLAIN
TSS_SECRET_MODE_POPUP = tss_lib.TSS_SECRET_MODE_POPUP
TSS_SECRET_MODE_CALLBACK = tss_lib.TSS_SECRET_MODE_CALLBACK

TSS_PS_TYPE_USER = tss_lib.TSS_PS_TYPE_USER
TSS_PS_TYPE_SYSTEM = tss_lib.TSS_PS_TYPE_SYSTEM

TSS_KEY_NO_AUTHORIZATION = tss_lib.TSS_KEY_NO_AUTHORIZATION
TSS_KEY_AUTHORIZATION = tss_lib.TSS_KEY_AUTHORIZATION
TSS_KEY_AUTHORIZATION_PRIV_USE_ONLY = tss_lib.TSS_KEY_AUTHORIZATION_PRIV_USE_ONLY
TSS_KEY_NON_VOLATILE = tss_lib.TSS_KEY_NON_VOLATILE
TSS_KEY_VOLATILE = tss_lib.TSS_KEY_VOLATILE
TSS_KEY_NOT_MIGRATABLE = tss_lib.TSS_KEY_NOT_MIGRATABLE
TSS_KEY_MIGRATABLE = tss_lib.TSS_KEY_MIGRATABLE
TSS_KEY_TYPE_DEFAULT = tss_lib.TSS_KEY_TYPE_DEFAULT
TSS_KEY_TYPE_SIGNING = tss_lib.TSS_KEY_TYPE_SIGNING
TSS_KEY_TYPE_STORAGE = tss_lib.TSS_KEY_TYPE_STORAGE
TSS_KEY_TYPE_IDENTITY = tss_lib.TSS_KEY_TYPE_IDENTITY
TSS_KEY_TYPE_AUTHCHANGE = tss_lib.TSS_KEY_TYPE_AUTHCHANGE
TSS_KEY_TYPE_BIND = tss_lib.TSS_KEY_TYPE_BIND
TSS_KEY_TYPE_LEGACY = tss_lib.TSS_KEY_TYPE_LEGACY
TSS_KEY_TYPE_MIGRATE = tss_lib.TSS_KEY_TYPE_MIGRATE
TSS_KEY_TYPE_BITMASK = tss_lib.TSS_KEY_TYPE_BITMASK
TSS_KEY_SIZE_DEFAULT = tss_lib.TSS_KEY_SIZE_DEFAULT
TSS_KEY_SIZE_512 = tss_lib.TSS_KEY_SIZE_512
TSS_KEY_SIZE_1024 = tss_lib.TSS_KEY_SIZE_1024
TSS_KEY_SIZE_2048 = tss_lib.TSS_KEY_SIZE_2048
TSS_KEY_SIZE_4096 = tss_lib.TSS_KEY_SIZE_4096
TSS_KEY_SIZE_8192 = tss_lib.TSS_KEY_SIZE_8192
TSS_KEY_SIZE_16384 = tss_lib.TSS_KEY_SIZE_16384
TSS_KEY_SIZE_BITMASK = tss_lib.TSS_KEY_SIZE_BITMASK
TSS_KEY_NOT_CERTIFIED_MIGRATABLE = tss_lib.TSS_KEY_NOT_CERTIFIED_MIGRATABLE
TSS_KEY_CERTIFIED_MIGRATABLE = tss_lib.TSS_KEY_CERTIFIED_MIGRATABLE
TSS_KEY_STRUCT_DEFAULT = tss_lib.TSS_KEY_STRUCT_DEFAULT
TSS_KEY_STRUCT_KEY = tss_lib.TSS_KEY_STRUCT_KEY
TSS_KEY_STRUCT_KEY12 = tss_lib.TSS_KEY_STRUCT_KEY12
TSS_KEY_STRUCT_BITMASK = tss_lib.TSS_KEY_STRUCT_BITMASK
TSS_KEY_EMPTY_KEY = tss_lib.TSS_KEY_EMPTY_KEY
TSS_KEY_TSP_SRK = tss_lib.TSS_KEY_TSP_SRK
TSS_KEY_TEMPLATE_BITMASK = tss_lib.TSS_KEY_TEMPLATE_BITMASK
TSS_KEY_SIZEVAL_512BIT = tss_lib.TSS_KEY_SIZEVAL_512BIT
TSS_KEY_SIZEVAL_1024BIT = tss_lib.TSS_KEY_SIZEVAL_1024BIT
TSS_KEY_SIZEVAL_2048BIT = tss_lib.TSS_KEY_SIZEVAL_2048BIT
TSS_KEY_SIZEVAL_4096BIT = tss_lib.TSS_KEY_SIZEVAL_4096BIT
TSS_KEY_SIZEVAL_8192BIT = tss_lib.TSS_KEY_SIZEVAL_8192BIT
TSS_KEY_SIZEVAL_16384BIT = tss_lib.TSS_KEY_SIZEVAL_16384BIT

TSS_TSPATTRIB_CONTEXT_SILENT_MODE = tss_lib.TSS_TSPATTRIB_CONTEXT_SILENT_MODE
TSS_TSPATTRIB_CONTEXT_MACHINE_NAME = tss_lib.TSS_TSPATTRIB_CONTEXT_MACHINE_NAME
TSS_TSPATTRIB_CONTEXT_VERSION_MODE = tss_lib.TSS_TSPATTRIB_CONTEXT_VERSION_MODE
TSS_TSPATTRIB_CONTEXT_TRANSPORT = tss_lib.TSS_TSPATTRIB_CONTEXT_TRANSPORT
TSS_TSPATTRIB_CONTEXT_CONNECTION_VERSION = tss_lib.TSS_TSPATTRIB_CONTEXT_CONNECTION_VERSION
TSS_TSPATTRIB_SECRET_HASH_MODE = tss_lib.TSS_TSPATTRIB_SECRET_HASH_MODE
TSS_TSPATTRIB_CONTEXTTRANS_CONTROL = tss_lib.TSS_TSPATTRIB_CONTEXTTRANS_CONTROL
TSS_TSPATTRIB_CONTEXTTRANS_MODE = tss_lib.TSS_TSPATTRIB_CONTEXTTRANS_MODE
TSS_TSPATTRIB_CONTEXT_NOT_SILENT = tss_lib.TSS_TSPATTRIB_CONTEXT_NOT_SILENT
TSS_TSPATTRIB_CONTEXT_SILENT = tss_lib.TSS_TSPATTRIB_CONTEXT_SILENT
TSS_TSPATTRIB_CONTEXT_VERSION_AUTO = tss_lib.TSS_TSPATTRIB_CONTEXT_VERSION_AUTO
TSS_TSPATTRIB_CONTEXT_VERSION_V1_1 = tss_lib.TSS_TSPATTRIB_CONTEXT_VERSION_V1_1
TSS_TSPATTRIB_CONTEXT_VERSION_V1_2 = tss_lib.TSS_TSPATTRIB_CONTEXT_VERSION_V1_2
TSS_TSPATTRIB_DISABLE_TRANSPORT = tss_lib.TSS_TSPATTRIB_DISABLE_TRANSPORT
TSS_TSPATTRIB_ENABLE_TRANSPORT = tss_lib.TSS_TSPATTRIB_ENABLE_TRANSPORT
TSS_TSPATTRIB_TRANSPORT_NO_DEFAULT_ENCRYPTION = tss_lib.TSS_TSPATTRIB_TRANSPORT_NO_DEFAULT_ENCRYPTION
TSS_TSPATTRIB_TRANSPORT_DEFAULT_ENCRYPTION = tss_lib.TSS_TSPATTRIB_TRANSPORT_DEFAULT_ENCRYPTION
TSS_TSPATTRIB_TRANSPORT_AUTHENTIC_CHANNEL = tss_lib.TSS_TSPATTRIB_TRANSPORT_AUTHENTIC_CHANNEL
TSS_TSPATTRIB_TRANSPORT_EXCLUSIVE = tss_lib.TSS_TSPATTRIB_TRANSPORT_EXCLUSIVE
TSS_TSPATTRIB_TRANSPORT_STATIC_AUTH = tss_lib.TSS_TSPATTRIB_TRANSPORT_STATIC_AUTH
TSS_TSPATTRIB_SECRET_HASH_MODE_POPUP = tss_lib.TSS_TSPATTRIB_SECRET_HASH_MODE_POPUP
TSS_TSPATTRIB_HASH_MODE_NOT_NULL = tss_lib.TSS_TSPATTRIB_HASH_MODE_NOT_NULL
TSS_TSPATTRIB_HASH_MODE_NULL = tss_lib.TSS_TSPATTRIB_HASH_MODE_NULL
TSS_TSPATTRIB_TPM_CALLBACK_COLLATEIDENTITY = tss_lib.TSS_TSPATTRIB_TPM_CALLBACK_COLLATEIDENTITY
TSS_TSPATTRIB_TPM_CALLBACK_ACTIVATEIDENTITY = tss_lib.TSS_TSPATTRIB_TPM_CALLBACK_ACTIVATEIDENTITY
TSS_TSPATTRIB_TPM_ORDINAL_AUDIT_STATUS = tss_lib.TSS_TSPATTRIB_TPM_ORDINAL_AUDIT_STATUS
TSS_TSPATTRIB_TPM_CREDENTIAL = tss_lib.TSS_TSPATTRIB_TPM_CREDENTIAL
TSS_TSPATTRIB_POLICY_CALLBACK_HMAC = tss_lib.TSS_TSPATTRIB_POLICY_CALLBACK_HMAC
TSS_TSPATTRIB_POLICY_CALLBACK_XOR_ENC = tss_lib.TSS_TSPATTRIB_POLICY_CALLBACK_XOR_ENC
TSS_TSPATTRIB_POLICY_CALLBACK_TAKEOWNERSHIP = tss_lib.TSS_TSPATTRIB_POLICY_CALLBACK_TAKEOWNERSHIP
TSS_TSPATTRIB_POLICY_CALLBACK_CHANGEAUTHASYM = tss_lib.TSS_TSPATTRIB_POLICY_CALLBACK_CHANGEAUTHASYM
TSS_TSPATTRIB_POLICY_SECRET_LIFETIME = tss_lib.TSS_TSPATTRIB_POLICY_SECRET_LIFETIME
TSS_TSPATTRIB_POLICY_POPUPSTRING = tss_lib.TSS_TSPATTRIB_POLICY_POPUPSTRING
TSS_TSPATTRIB_POLICY_CALLBACK_SEALX_MASK = tss_lib.TSS_TSPATTRIB_POLICY_CALLBACK_SEALX_MASK
TSS_TSPATTRIB_SECRET_HASH_MODE = tss_lib.TSS_TSPATTRIB_SECRET_HASH_MODE
TSS_TSPATTRIB_POLICY_DELEGATION_INFO = tss_lib.TSS_TSPATTRIB_POLICY_DELEGATION_INFO
TSS_TSPATTRIB_POLICY_DELEGATION_PCR = tss_lib.TSS_TSPATTRIB_POLICY_DELEGATION_PCR
TSS_TSPATTRIB_POLSECRET_LIFETIME_ALWAYS = tss_lib.TSS_TSPATTRIB_POLSECRET_LIFETIME_ALWAYS
TSS_TSPATTRIB_POLSECRET_LIFETIME_COUNTER = tss_lib.TSS_TSPATTRIB_POLSECRET_LIFETIME_COUNTER
TSS_TSPATTRIB_POLSECRET_LIFETIME_TIMER = tss_lib.TSS_TSPATTRIB_POLSECRET_LIFETIME_TIMER
TSS_TSPATTRIB_POLICYSECRET_LIFETIME_ALWAYS = tss_lib.TSS_TSPATTRIB_POLICYSECRET_LIFETIME_ALWAYS
TSS_TSPATTRIB_POLICYSECRET_LIFETIME_COUNTER = tss_lib.TSS_TSPATTRIB_POLICYSECRET_LIFETIME_COUNTER
TSS_TSPATTRIB_POLICYSECRET_LIFETIME_TIMER = tss_lib.TSS_TSPATTRIB_POLICYSECRET_LIFETIME_TIMER
TSS_TSPATTRIB_POLDEL_TYPE = tss_lib.TSS_TSPATTRIB_POLDEL_TYPE
TSS_TSPATTRIB_POLDEL_INDEX = tss_lib.TSS_TSPATTRIB_POLDEL_INDEX
TSS_TSPATTRIB_POLDEL_PER1 = tss_lib.TSS_TSPATTRIB_POLDEL_PER1
TSS_TSPATTRIB_POLDEL_PER2 = tss_lib.TSS_TSPATTRIB_POLDEL_PER2
TSS_TSPATTRIB_POLDEL_LABEL = tss_lib.TSS_TSPATTRIB_POLDEL_LABEL
TSS_TSPATTRIB_POLDEL_FAMILYID = tss_lib.TSS_TSPATTRIB_POLDEL_FAMILYID
TSS_TSPATTRIB_POLDEL_VERCOUNT = tss_lib.TSS_TSPATTRIB_POLDEL_VERCOUNT
TSS_TSPATTRIB_POLDEL_OWNERBLOB = tss_lib.TSS_TSPATTRIB_POLDEL_OWNERBLOB
TSS_TSPATTRIB_POLDEL_KEYBLOB = tss_lib.TSS_TSPATTRIB_POLDEL_KEYBLOB
TSS_TSPATTRIB_POLDELPCR_LOCALITY = tss_lib.TSS_TSPATTRIB_POLDELPCR_LOCALITY
TSS_TSPATTRIB_POLDELPCR_DIGESTATRELEASE = tss_lib.TSS_TSPATTRIB_POLDELPCR_DIGESTATRELEASE
TSS_TSPATTRIB_POLDELPCR_SELECTION = tss_lib.TSS_TSPATTRIB_POLDELPCR_SELECTION
TSS_TSPATTRIB_ENCDATA_BLOB = tss_lib.TSS_TSPATTRIB_ENCDATA_BLOB
TSS_TSPATTRIB_ENCDATA_PCR = tss_lib.TSS_TSPATTRIB_ENCDATA_PCR
TSS_TSPATTRIB_ENCDATA_PCR_LONG = tss_lib.TSS_TSPATTRIB_ENCDATA_PCR_LONG
TSS_TSPATTRIB_ENCDATA_SEAL = tss_lib.TSS_TSPATTRIB_ENCDATA_SEAL
TSS_TSPATTRIB_ENCDATABLOB_BLOB = tss_lib.TSS_TSPATTRIB_ENCDATABLOB_BLOB
TSS_TSPATTRIB_ENCDATAPCR_DIGEST_ATCREATION = tss_lib.TSS_TSPATTRIB_ENCDATAPCR_DIGEST_ATCREATION
TSS_TSPATTRIB_ENCDATAPCR_DIGEST_ATRELEASE = tss_lib.TSS_TSPATTRIB_ENCDATAPCR_DIGEST_ATRELEASE
TSS_TSPATTRIB_ENCDATAPCR_SELECTION = tss_lib.TSS_TSPATTRIB_ENCDATAPCR_SELECTION
TSS_TSPATTRIB_ENCDATAPCR_DIGEST_RELEASE = tss_lib.TSS_TSPATTRIB_ENCDATAPCR_DIGEST_RELEASE
TSS_TSPATTRIB_ENCDATAPCRLONG_LOCALITY_ATCREATION = tss_lib.TSS_TSPATTRIB_ENCDATAPCRLONG_LOCALITY_ATCREATION
TSS_TSPATTRIB_ENCDATAPCRLONG_LOCALITY_ATRELEASE = tss_lib.TSS_TSPATTRIB_ENCDATAPCRLONG_LOCALITY_ATRELEASE
TSS_TSPATTRIB_ENCDATAPCRLONG_CREATION_SELECTION = tss_lib.TSS_TSPATTRIB_ENCDATAPCRLONG_CREATION_SELECTION
TSS_TSPATTRIB_ENCDATAPCRLONG_RELEASE_SELECTION = tss_lib.TSS_TSPATTRIB_ENCDATAPCRLONG_RELEASE_SELECTION
TSS_TSPATTRIB_ENCDATAPCRLONG_DIGEST_ATCREATION = tss_lib.TSS_TSPATTRIB_ENCDATAPCRLONG_DIGEST_ATCREATION
TSS_TSPATTRIB_ENCDATAPCRLONG_DIGEST_ATRELEASE = tss_lib.TSS_TSPATTRIB_ENCDATAPCRLONG_DIGEST_ATRELEASE
TSS_TSPATTRIB_ENCDATASEAL_PROTECT_MODE = tss_lib.TSS_TSPATTRIB_ENCDATASEAL_PROTECT_MODE
TSS_TSPATTRIB_ENCDATASEAL_NOPROTECT = tss_lib.TSS_TSPATTRIB_ENCDATASEAL_NOPROTECT
TSS_TSPATTRIB_ENCDATASEAL_PROTECT = tss_lib.TSS_TSPATTRIB_ENCDATASEAL_PROTECT
TSS_TSPATTRIB_ENCDATASEAL_NO_PROTECT = tss_lib.TSS_TSPATTRIB_ENCDATASEAL_NO_PROTECT
TSS_TSPATTRIB_NV_INDEX = tss_lib.TSS_TSPATTRIB_NV_INDEX
TSS_TSPATTRIB_NV_PERMISSIONS = tss_lib.TSS_TSPATTRIB_NV_PERMISSIONS
TSS_TSPATTRIB_NV_STATE = tss_lib.TSS_TSPATTRIB_NV_STATE
TSS_TSPATTRIB_NV_DATASIZE = tss_lib.TSS_TSPATTRIB_NV_DATASIZE
TSS_TSPATTRIB_NV_PCR = tss_lib.TSS_TSPATTRIB_NV_PCR
TSS_TSPATTRIB_NVSTATE_READSTCLEAR = tss_lib.TSS_TSPATTRIB_NVSTATE_READSTCLEAR
TSS_TSPATTRIB_NVSTATE_WRITESTCLEAR = tss_lib.TSS_TSPATTRIB_NVSTATE_WRITESTCLEAR
TSS_TSPATTRIB_NVSTATE_WRITEDEFINE = tss_lib.TSS_TSPATTRIB_NVSTATE_WRITEDEFINE
TSS_TSPATTRIB_NVPCR_READPCRSELECTION = tss_lib.TSS_TSPATTRIB_NVPCR_READPCRSELECTION
TSS_TSPATTRIB_NVPCR_READDIGESTATRELEASE = tss_lib.TSS_TSPATTRIB_NVPCR_READDIGESTATRELEASE
TSS_TSPATTRIB_NVPCR_READLOCALITYATRELEASE = tss_lib.TSS_TSPATTRIB_NVPCR_READLOCALITYATRELEASE
TSS_TSPATTRIB_NVPCR_WRITEPCRSELECTION = tss_lib.TSS_TSPATTRIB_NVPCR_WRITEPCRSELECTION
TSS_TSPATTRIB_NVPCR_WRITEDIGESTATRELEASE = tss_lib.TSS_TSPATTRIB_NVPCR_WRITEDIGESTATRELEASE
TSS_TSPATTRIB_NVPCR_WRITELOCALITYATRELEASE = tss_lib.TSS_TSPATTRIB_NVPCR_WRITELOCALITYATRELEASE
TSS_TSPATTRIB_HASH_IDENTIFIER = tss_lib.TSS_TSPATTRIB_HASH_IDENTIFIER
TSS_TSPATTRIB_ALG_IDENTIFIER = tss_lib.TSS_TSPATTRIB_ALG_IDENTIFIER
TSS_TSPATTRIB_PCRS_INFO = tss_lib.TSS_TSPATTRIB_PCRS_INFO
TSS_TSPATTRIB_PCRSINFO_PCRSTRUCT = tss_lib.TSS_TSPATTRIB_PCRSINFO_PCRSTRUCT
TSS_TSPATTRIB_DELFAMILY_STATE = tss_lib.TSS_TSPATTRIB_DELFAMILY_STATE
TSS_TSPATTRIB_DELFAMILY_INFO = tss_lib.TSS_TSPATTRIB_DELFAMILY_INFO
TSS_TSPATTRIB_DELFAMILYSTATE_LOCKED = tss_lib.TSS_TSPATTRIB_DELFAMILYSTATE_LOCKED
TSS_TSPATTRIB_DELFAMILYSTATE_ENABLED = tss_lib.TSS_TSPATTRIB_DELFAMILYSTATE_ENABLED
TSS_TSPATTRIB_DELFAMILYINFO_LABEL = tss_lib.TSS_TSPATTRIB_DELFAMILYINFO_LABEL
TSS_TSPATTRIB_DELFAMILYINFO_VERCOUNT = tss_lib.TSS_TSPATTRIB_DELFAMILYINFO_VERCOUNT
TSS_TSPATTRIB_DELFAMILYINFO_FAMILYID = tss_lib.TSS_TSPATTRIB_DELFAMILYINFO_FAMILYID
TSS_TSPATTRIB_DAACRED_COMMIT = tss_lib.TSS_TSPATTRIB_DAACRED_COMMIT
TSS_TSPATTRIB_DAACRED_ATTRIB_GAMMAS = tss_lib.TSS_TSPATTRIB_DAACRED_ATTRIB_GAMMAS
TSS_TSPATTRIB_DAACRED_CREDENTIAL_BLOB = tss_lib.TSS_TSPATTRIB_DAACRED_CREDENTIAL_BLOB
TSS_TSPATTRIB_DAACRED_CALLBACK_SIGN = tss_lib.TSS_TSPATTRIB_DAACRED_CALLBACK_SIGN
TSS_TSPATTRIB_DAACRED_CALLBACK_VERIFYSIGNATURE = tss_lib.TSS_TSPATTRIB_DAACRED_CALLBACK_VERIFYSIGNATURE
TSS_TSPATTRIB_DAACOMMIT_NUMBER = tss_lib.TSS_TSPATTRIB_DAACOMMIT_NUMBER
TSS_TSPATTRIB_DAACOMMIT_SELECTION = tss_lib.TSS_TSPATTRIB_DAACOMMIT_SELECTION
TSS_TSPATTRIB_DAACOMMIT_COMMITMENTS = tss_lib.TSS_TSPATTRIB_DAACOMMIT_COMMITMENTS
TSS_TSPATTRIB_DAAATTRIBGAMMAS_BLOB = tss_lib.TSS_TSPATTRIB_DAAATTRIBGAMMAS_BLOB
TSS_TSPATTRIB_DAAISSUERKEY_BLOB = tss_lib.TSS_TSPATTRIB_DAAISSUERKEY_BLOB
TSS_TSPATTRIB_DAAISSUERKEY_PUBKEY = tss_lib.TSS_TSPATTRIB_DAAISSUERKEY_PUBKEY
TSS_TSPATTRIB_DAAISSUERKEYBLOB_PUBLIC_KEY = tss_lib.TSS_TSPATTRIB_DAAISSUERKEYBLOB_PUBLIC_KEY
TSS_TSPATTRIB_DAAISSUERKEYBLOB_SECRET_KEY = tss_lib.TSS_TSPATTRIB_DAAISSUERKEYBLOB_SECRET_KEY
TSS_TSPATTRIB_DAAISSUERKEYBLOB_KEYBLOB = tss_lib.TSS_TSPATTRIB_DAAISSUERKEYBLOB_KEYBLOB
TSS_TSPATTRIB_DAAISSUERKEYBLOB_PROOF = tss_lib.TSS_TSPATTRIB_DAAISSUERKEYBLOB_PROOF
TSS_TSPATTRIB_DAAISSUERKEYPUBKEY_NUM_ATTRIBS = tss_lib.TSS_TSPATTRIB_DAAISSUERKEYPUBKEY_NUM_ATTRIBS
TSS_TSPATTRIB_DAAISSUERKEYPUBKEY_NUM_PLATFORM_ATTRIBS = tss_lib.TSS_TSPATTRIB_DAAISSUERKEYPUBKEY_NUM_PLATFORM_ATTRIBS
TSS_TSPATTRIB_DAAISSUERKEYPUBKEY_NUM_ISSUER_ATTRIBS = tss_lib.TSS_TSPATTRIB_DAAISSUERKEYPUBKEY_NUM_ISSUER_ATTRIBS
TSS_TSPATTRIB_DAAARAKEY_BLOB = tss_lib.TSS_TSPATTRIB_DAAARAKEY_BLOB
TSS_TSPATTRIB_DAAARAKEYBLOB_PUBLIC_KEY = tss_lib.TSS_TSPATTRIB_DAAARAKEYBLOB_PUBLIC_KEY
TSS_TSPATTRIB_DAAARAKEYBLOB_SECRET_KEY = tss_lib.TSS_TSPATTRIB_DAAARAKEYBLOB_SECRET_KEY
TSS_TSPATTRIB_DAAARAKEYBLOB_KEYBLOB = tss_lib.TSS_TSPATTRIB_DAAARAKEYBLOB_KEYBLOB
TSS_TSPATTRIB_KEY_BLOB = tss_lib.TSS_TSPATTRIB_KEY_BLOB
TSS_TSPATTRIB_KEY_INFO = tss_lib.TSS_TSPATTRIB_KEY_INFO
TSS_TSPATTRIB_KEY_UUID = tss_lib.TSS_TSPATTRIB_KEY_UUID
TSS_TSPATTRIB_KEY_PCR = tss_lib.TSS_TSPATTRIB_KEY_PCR
TSS_TSPATTRIB_RSAKEY_INFO = tss_lib.TSS_TSPATTRIB_RSAKEY_INFO
TSS_TSPATTRIB_KEY_REGISTER = tss_lib.TSS_TSPATTRIB_KEY_REGISTER
TSS_TSPATTRIB_KEY_PCR_LONG = tss_lib.TSS_TSPATTRIB_KEY_PCR_LONG
TSS_TSPATTRIB_KEY_CONTROLBIT = tss_lib.TSS_TSPATTRIB_KEY_CONTROLBIT
TSS_TSPATTRIB_KEY_CMKINFO = tss_lib.TSS_TSPATTRIB_KEY_CMKINFO
TSS_TSPATTRIB_KEYBLOB_BLOB = tss_lib.TSS_TSPATTRIB_KEYBLOB_BLOB
TSS_TSPATTRIB_KEYBLOB_PUBLIC_KEY = tss_lib.TSS_TSPATTRIB_KEYBLOB_PUBLIC_KEY
TSS_TSPATTRIB_KEYBLOB_PRIVATE_KEY = tss_lib.TSS_TSPATTRIB_KEYBLOB_PRIVATE_KEY
TSS_TSPATTRIB_KEYINFO_SIZE = tss_lib.TSS_TSPATTRIB_KEYINFO_SIZE
TSS_TSPATTRIB_KEYINFO_USAGE = tss_lib.TSS_TSPATTRIB_KEYINFO_USAGE
TSS_TSPATTRIB_KEYINFO_KEYFLAGS = tss_lib.TSS_TSPATTRIB_KEYINFO_KEYFLAGS
TSS_TSPATTRIB_KEYINFO_AUTHUSAGE = tss_lib.TSS_TSPATTRIB_KEYINFO_AUTHUSAGE
TSS_TSPATTRIB_KEYINFO_ALGORITHM = tss_lib.TSS_TSPATTRIB_KEYINFO_ALGORITHM
TSS_TSPATTRIB_KEYINFO_SIGSCHEME = tss_lib.TSS_TSPATTRIB_KEYINFO_SIGSCHEME
TSS_TSPATTRIB_KEYINFO_ENCSCHEME = tss_lib.TSS_TSPATTRIB_KEYINFO_ENCSCHEME
TSS_TSPATTRIB_KEYINFO_MIGRATABLE = tss_lib.TSS_TSPATTRIB_KEYINFO_MIGRATABLE
TSS_TSPATTRIB_KEYINFO_REDIRECTED = tss_lib.TSS_TSPATTRIB_KEYINFO_REDIRECTED
TSS_TSPATTRIB_KEYINFO_VOLATILE = tss_lib.TSS_TSPATTRIB_KEYINFO_VOLATILE
TSS_TSPATTRIB_KEYINFO_AUTHDATAUSAGE = tss_lib.TSS_TSPATTRIB_KEYINFO_AUTHDATAUSAGE
TSS_TSPATTRIB_KEYINFO_VERSION = tss_lib.TSS_TSPATTRIB_KEYINFO_VERSION
TSS_TSPATTRIB_KEYINFO_CMK = tss_lib.TSS_TSPATTRIB_KEYINFO_CMK
TSS_TSPATTRIB_KEYINFO_KEYSTRUCT = tss_lib.TSS_TSPATTRIB_KEYINFO_KEYSTRUCT
TSS_TSPATTRIB_KEYCONTROL_OWNEREVICT = tss_lib.TSS_TSPATTRIB_KEYCONTROL_OWNEREVICT
TSS_TSPATTRIB_KEYINFO_RSA_EXPONENT = tss_lib.TSS_TSPATTRIB_KEYINFO_RSA_EXPONENT
TSS_TSPATTRIB_KEYINFO_RSA_MODULUS = tss_lib.TSS_TSPATTRIB_KEYINFO_RSA_MODULUS
TSS_TSPATTRIB_KEYINFO_RSA_KEYSIZE = tss_lib.TSS_TSPATTRIB_KEYINFO_RSA_KEYSIZE
TSS_TSPATTRIB_KEYINFO_RSA_PRIMES = tss_lib.TSS_TSPATTRIB_KEYINFO_RSA_PRIMES
TSS_TSPATTRIB_KEYPCR_DIGEST_ATCREATION = tss_lib.TSS_TSPATTRIB_KEYPCR_DIGEST_ATCREATION
TSS_TSPATTRIB_KEYPCR_DIGEST_ATRELEASE = tss_lib.TSS_TSPATTRIB_KEYPCR_DIGEST_ATRELEASE
TSS_TSPATTRIB_KEYPCR_SELECTION = tss_lib.TSS_TSPATTRIB_KEYPCR_SELECTION
TSS_TSPATTRIB_KEYREGISTER_USER = tss_lib.TSS_TSPATTRIB_KEYREGISTER_USER
TSS_TSPATTRIB_KEYREGISTER_SYSTEM = tss_lib.TSS_TSPATTRIB_KEYREGISTER_SYSTEM
TSS_TSPATTRIB_KEYREGISTER_NO = tss_lib.TSS_TSPATTRIB_KEYREGISTER_NO
TSS_TSPATTRIB_KEYPCRLONG_LOCALITY_ATCREATION = tss_lib.TSS_TSPATTRIB_KEYPCRLONG_LOCALITY_ATCREATION
TSS_TSPATTRIB_KEYPCRLONG_LOCALITY_ATRELEASE = tss_lib.TSS_TSPATTRIB_KEYPCRLONG_LOCALITY_ATRELEASE
TSS_TSPATTRIB_KEYPCRLONG_CREATION_SELECTION = tss_lib.TSS_TSPATTRIB_KEYPCRLONG_CREATION_SELECTION
TSS_TSPATTRIB_KEYPCRLONG_RELEASE_SELECTION = tss_lib.TSS_TSPATTRIB_KEYPCRLONG_RELEASE_SELECTION
TSS_TSPATTRIB_KEYPCRLONG_DIGEST_ATCREATION = tss_lib.TSS_TSPATTRIB_KEYPCRLONG_DIGEST_ATCREATION
TSS_TSPATTRIB_KEYPCRLONG_DIGEST_ATRELEASE = tss_lib.TSS_TSPATTRIB_KEYPCRLONG_DIGEST_ATRELEASE
TSS_TSPATTRIB_KEYINFO_CMK_MA_APPROVAL = tss_lib.TSS_TSPATTRIB_KEYINFO_CMK_MA_APPROVAL
TSS_TSPATTRIB_KEYINFO_CMK_MA_DIGEST = tss_lib.TSS_TSPATTRIB_KEYINFO_CMK_MA_DIGEST

TPM_ALG_RSA = tss_lib.TPM_ALG_RSA
TPM_ALG_DES = tss_lib.TPM_ALG_DES
TPM_ALG_3DES = tss_lib.TPM_ALG_3DES
TPM_ALG_SHA = tss_lib.TPM_ALG_SHA
TPM_ALG_HMAC = tss_lib.TPM_ALG_HMAC
TPM_ALG_AES = tss_lib.TPM_ALG_AES
TPM_ALG_AES128 = tss_lib.TPM_ALG_AES128
TPM_ALG_MGF1 = tss_lib.TPM_ALG_MGF1
TPM_ALG_AES192 = tss_lib.TPM_ALG_AES192
TPM_ALG_AES256 = tss_lib.TPM_ALG_AES256
TPM_ALG_XOR = tss_lib.TPM_ALG_XOR

TPM_ES_NONE = tss_lib.TPM_ES_NONE
TPM_ES_RSAESPKCSv15 = tss_lib.TPM_ES_RSAESPKCSv15
TPM_ES_RSAESOAEP_SHA1_MGF1 = tss_lib.TPM_ES_RSAESOAEP_SHA1_MGF1
TPM_ES_SYM_CNT = tss_lib.TPM_ES_SYM_CNT
TPM_ES_SYM_CTR = tss_lib.TPM_ES_SYM_CTR
TPM_ES_SYM_OFB = tss_lib.TPM_ES_SYM_OFB
TPM_ES_SYM_CBC_PKCS5PAD = tss_lib.TPM_ES_SYM_CBC_PKCS5PAD

TPM_SS_NONE = tss_lib.TPM_SS_NONE
TPM_SS_RSASSAPKCS1v15_SHA1 = tss_lib.TPM_SS_RSASSAPKCS1v15_SHA1
TPM_SS_RSASSAPKCS1v15_DER = tss_lib.TPM_SS_RSASSAPKCS1v15_DER
TPM_SS_RSASSAPKCS1v15_INFO = tss_lib.TPM_SS_RSASSAPKCS1v15_INFO

TSS_PCRS_STRUCT_DEFAULT = tss_lib.TSS_PCRS_STRUCT_DEFAULT
TSS_PCRS_STRUCT_INFO = tss_lib.TSS_PCRS_STRUCT_INFO
TSS_PCRS_STRUCT_INFO_LONG = tss_lib.TSS_PCRS_STRUCT_INFO_LONG
TSS_PCRS_STRUCT_INFO_SHORT = tss_lib.TSS_PCRS_STRUCT_INFO_SHORT
TSS_PCRS_DIRECTION_CREATION = tss_lib.TSS_PCRS_DIRECTION_CREATION
TSS_PCRS_DIRECTION_RELEASE = tss_lib.TSS_PCRS_DIRECTION_RELEASE

TSS_HASH_SHA1 = tss_lib.TSS_HASH_SHA1
TSS_HASH_OTHER = tss_lib.TSS_HASH_OTHER



class TspiException(Exception):
    pass

class TSS_E_BASE(TspiException):
    pass

class TSS_E_FAIL(TspiException):
    pass

class TSS_E_BAD_PARAMETER(TspiException):
    pass

class TSS_E_INTERNAL_ERROR(TspiException):
    pass

class TSS_E_OUTOFMEMORY(TspiException):
    pass

class TSS_E_NOTIMPL(TspiException):
    pass

class TSS_E_KEY_ALREADY_REGISTERED(TspiException):
    pass

class TSS_E_TPM_UNEXPECTED(TspiException):
    pass

class TSS_E_COMM_FAILURE(TspiException):
    pass

class TSS_E_TIMEOUT(TspiException):
    pass

class TSS_E_TPM_UNSUPPORTED_FEATURE(TspiException):
    pass

class TSS_E_CANCELED(TspiException):
    pass

class TSS_E_PS_KEY_NOTFOUND(TspiException):
    pass

class TSS_E_PS_KEY_EXISTS(TspiException):
    pass

class TSS_E_PS_BAD_KEY_STATE(TspiException):
    pass

class TSS_E_INVALID_OBJECT_TYPE(TspiException):
    pass

class TSS_E_NO_CONNECTION(TspiException):
    pass

class TSS_E_CONNECTION_FAILED(TspiException):
    pass

class TSS_E_CONNECTION_BROKEN(TspiException):
    pass

class TSS_E_HASH_INVALID_ALG(TspiException):
    pass

class TSS_E_HASH_INVALID_LENGTH(TspiException):
    pass

class TSS_E_HASH_NO_DATA(TspiException):
    pass

class TSS_E_INVALID_ATTRIB_FLAG(TspiException):
    pass

class TSS_E_INVALID_ATTRIB_SUBFLAG(TspiException):
    pass

class TSS_E_INVALID_ATTRIB_DATA(TspiException):
    pass

class TSS_E_INVALID_OBJECT_INIT_FLAG(TspiException):
    pass

class TSS_E_INVALID_OBJECT_INITFLAG(TspiException):
    pass

class TSS_E_NO_PCRS_SET(TspiException):
    pass

class TSS_E_KEY_NOT_LOADED(TspiException):
    pass

class TSS_E_KEY_NOT_SET(TspiException):
    pass

class TSS_E_VALIDATION_FAILED(TspiException):
    pass

class TSS_E_TSP_AUTHREQUIRED(TspiException):
    pass

class TSS_E_TSP_AUTH2REQUIRED(TspiException):
    pass

class TSS_E_TSP_AUTHFAIL(TspiException):
    pass

class TSS_E_TSP_AUTH2FAIL(TspiException):
    pass

class TSS_E_KEY_NO_MIGRATION_POLICY(TspiException):
    pass

class TSS_E_POLICY_NO_SECRET(TspiException):
    pass

class TSS_E_INVALID_OBJ_ACCESS(TspiException):
    pass

class TSS_E_INVALID_ENCSCHEME(TspiException):
    pass

class TSS_E_INVALID_SIGSCHEME(TspiException):
    pass

class TSS_E_ENC_INVALID_LENGTH(TspiException):
    pass

class TSS_E_ENC_NO_DATA(TspiException):
    pass

class TSS_E_ENC_INVALID_TYPE(TspiException):
    pass

class TSS_E_INVALID_KEYUSAGE(TspiException):
    pass

class TSS_E_VERIFICATION_FAILED(TspiException):
    pass

class TSS_E_HASH_NO_IDENTIFIER(TspiException):
    pass

class TSS_E_INVALID_HANDLE(TspiException):
    pass

class TSS_E_SILENT_CONTEXT(TspiException):
    pass

class TSS_E_EK_CHECKSUM(TspiException):
    pass

class TSS_E_DELEGATION_NOTSET(TspiException):
    pass

class TSS_E_DELFAMILY_NOTFOUND(TspiException):
    pass

class TSS_E_DELFAMILY_ROWEXISTS(TspiException):
    pass

class TSS_E_VERSION_MISMATCH(TspiException):
    pass

class TSS_E_DAA_AR_DECRYPTION_ERROR(TspiException):
    pass

class TSS_E_DAA_AUTHENTICATION_ERROR(TspiException):
    pass

class TSS_E_DAA_CHALLENGE_RESPONSE_ERROR(TspiException):
    pass

class TSS_E_DAA_CREDENTIAL_PROOF_ERROR(TspiException):
    pass

class TSS_E_DAA_CREDENTIAL_REQUEST_PROOF_ERROR(TspiException):
    pass

class TSS_E_DAA_ISSUER_KEY_ERROR(TspiException):
    pass

class TSS_E_DAA_PSEUDONYM_ERROR(TspiException):
    pass

class TSS_E_INVALID_RESOURCE(TspiException):
    pass

class TSS_E_NV_AREA_EXIST(TspiException):
    pass

class TSS_E_NV_AREA_NOT_EXIST(TspiException):
    pass

class TSS_E_TSP_TRANS_AUTHFAIL(TspiException):
    pass

class TSS_E_TSP_TRANS_AUTHREQUIRED(TspiException):
    pass

class TSS_E_TSP_TRANS_NOTEXCLUSIVE(TspiException):
    pass

class TSS_E_TSP_TRANS_FAIL(TspiException):
    pass

class TSS_E_TSP_TRANS_NO_PUBKEY(TspiException):
    pass

class TSS_E_NO_ACTIVE_COUNTER(TspiException):
    pass

class TpmException(Exception):
    pass

class TPM_E_BASE(TpmException):
    pass

class TPM_E_NON_FATAL(TpmException):
    pass

class TPM_E_AUTHFAIL(TpmException):
    pass

class TPM_E_BADINDEX(TpmException):
    pass

class TPM_E_BAD_PARAMETER(TpmException):
    pass

class TPM_E_AUDITFAILURE(TpmException):
    pass

class TPM_E_CLEAR_DISABLED(TpmException):
    pass

class TPM_E_DEACTIVATED(TpmException):
    pass

class TPM_E_DISABLED(TpmException):
    pass

class TPM_E_DISABLED_CMD(TpmException):
    pass

class TPM_E_FAIL(TpmException):
    pass

class TPM_E_BAD_ORDINAL(TpmException):
    pass

class TPM_E_INSTALL_DISABLED(TpmException):
    pass

class TPM_E_INVALID_KEYHANDLE(TpmException):
    pass

class TPM_E_KEYNOTFOUND(TpmException):
    pass

class TPM_E_INAPPROPRIATE_ENC(TpmException):
    pass

class TPM_E_MIGRATEFAIL(TpmException):
    pass

class TPM_E_INVALID_PCR_INFO(TpmException):
    pass

class TPM_E_NOSPACE(TpmException):
    pass

class TPM_E_NOSRK(TpmException):
    pass

class TPM_E_NOTSEALED_BLOB(TpmException):
    pass

class TPM_E_OWNER_SET(TpmException):
    pass

class TPM_E_RESOURCES(TpmException):
    pass

class TPM_E_SHORTRANDOM(TpmException):
    pass

class TPM_E_SIZE(TpmException):
    pass

class TPM_E_WRONGPCRVAL(TpmException):
    pass

class TPM_E_BAD_PARAM_SIZE(TpmException):
    pass

class TPM_E_SHA_THREAD(TpmException):
    pass

class TPM_E_SHA_ERROR(TpmException):
    pass

class TPM_E_FAILEDSELFTEST(TpmException):
    pass

class TPM_E_AUTH2FAIL(TpmException):
    pass

class TPM_E_BADTAG(TpmException):
    pass

class TPM_E_IOERROR(TpmException):
    pass

class TPM_E_ENCRYPT_ERROR(TpmException):
    pass

class TPM_E_DECRYPT_ERROR(TpmException):
    pass

class TPM_E_INVALID_AUTHHANDLE(TpmException):
    pass

class TPM_E_NO_ENDORSEMENT(TpmException):
    pass

class TPM_E_INVALID_KEYUSAGE(TpmException):
    pass

class TPM_E_WRONG_ENTITYTYPE(TpmException):
    pass

class TPM_E_INVALID_POSTINIT(TpmException):
    pass

class TPM_E_INAPPROPRIATE_SIG(TpmException):
    pass

class TPM_E_BAD_KEY_PROPERTY(TpmException):
    pass

class TPM_E_BAD_MIGRATION(TpmException):
    pass

class TPM_E_BAD_SCHEME(TpmException):
    pass

class TPM_E_BAD_DATASIZE(TpmException):
    pass

class TPM_E_BAD_MODE(TpmException):
    pass

class TPM_E_BAD_PRESENCE(TpmException):
    pass

class TPM_E_BAD_VERSION(TpmException):
    pass

class TPM_E_NO_WRAP_TRANSPORT(TpmException):
    pass

class TPM_E_AUDITFAIL_UNSUCCESSFUL(TpmException):
    pass

class TPM_E_AUDITFAIL_SUCCESSFUL(TpmException):
    pass

class TPM_E_NOTRESETABLE(TpmException):
    pass

class TPM_E_NOTLOCAL(TpmException):
    pass

class TPM_E_BAD_TYPE(TpmException):
    pass

class TPM_E_INVALID_RESOURCE(TpmException):
    pass

class TPM_E_NOTFIPS(TpmException):
    pass

class TPM_E_INVALID_FAMILY(TpmException):
    pass

class TPM_E_NO_NV_PERMISSION(TpmException):
    pass

class TPM_E_REQUIRES_SIGN(TpmException):
    pass

class TPM_E_KEY_NOTSUPPORTED(TpmException):
    pass

class TPM_E_AUTH_CONFLICT(TpmException):
    pass

class TPM_E_AREA_LOCKED(TpmException):
    pass

class TPM_E_BAD_LOCALITY(TpmException):
    pass

class TPM_E_READ_ONLY(TpmException):
    pass

class TPM_E_PER_NOWRITE(TpmException):
    pass

class TPM_E_FAMILYCOUNT(TpmException):
    pass

class TPM_E_WRITE_LOCKED(TpmException):
    pass

class TPM_E_BAD_ATTRIBUTES(TpmException):
    pass

class TPM_E_INVALID_STRUCTURE(TpmException):
    pass

class TPM_E_KEY_OWNER_CONTROL(TpmException):
    pass

class TPM_E_BAD_COUNTER(TpmException):
    pass

class TPM_E_NOT_FULLWRITE(TpmException):
    pass

class TPM_E_CONTEXT_GAP(TpmException):
    pass

class TPM_E_MAXNVWRITES(TpmException):
    pass

class TPM_E_NOOPERATOR(TpmException):
    pass

class TPM_E_RESOURCEMISSING(TpmException):
    pass

class TPM_E_DELEGATE_LOCK(TpmException):
    pass

class TPM_E_DELEGATE_FAMILY(TpmException):
    pass

class TPM_E_DELEGATE_ADMIN(TpmException):
    pass

class TPM_E_TRANSPORT_NOTEXCLUSIVE(TpmException):
    pass

class TPM_E_OWNER_CONTROL(TpmException):
    pass

class TPM_E_DAA_RESOURCES(TpmException):
    pass

class TPM_E_DAA_INPUT_DATA0(TpmException):
    pass

class TPM_E_DAA_INPUT_DATA1(TpmException):
    pass

class TPM_E_DAA_ISSUER_SETTINGS(TpmException):
    pass

class TPM_E_DAA_TPM_SETTINGS(TpmException):
    pass

class TPM_E_DAA_STAGE(TpmException):
    pass

class TPM_E_DAA_ISSUER_VALIDITY(TpmException):
    pass

class TPM_E_DAA_WRONG_W(TpmException):
    pass

class TPM_E_BAD_HANDLE(TpmException):
    pass

class TPM_E_BAD_DELEGATE(TpmException):
    pass

class TPM_E_BADCONTEXT(TpmException):
    pass

class TPM_E_TOOMANYCONTEXTS(TpmException):
    pass

class TPM_E_MA_TICKET_SIGNATURE(TpmException):
    pass

class TPM_E_MA_DESTINATION(TpmException):
    pass

class TPM_E_MA_SOURCE(TpmException):
    pass

class TPM_E_MA_AUTHORITY(TpmException):
    pass

class TPM_E_PERMANENTEK(TpmException):
    pass

class TPM_E_BAD_SIGNATURE(TpmException):
    pass

class TPM_E_NOCONTEXTSPACE(TpmException):
    pass

class TPM_E_RETRY(TpmException):
    pass

class TPM_E_NEEDS_SELFTEST(TpmException):
    pass

class TPM_E_DOING_SELFTEST(TpmException):
    pass

class TPM_E_DEFEND_LOCK_RUNNING(TpmException):
    pass



#!/usr/bin/env python
"""
pytss is a python language wrapper for the Tspi library provided by the
TrouSerS project. libtspi is a library for interfacing with a TPM services
daemon.

Author: Matthew Garrett (matthew.garrett@nebula.com)

Copyright 2013 Nebula, Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from setuptools import setup

__about__ = {}

with open('pytss/__about__.py') as fp:
    exec(fp.read(), None, __about__)


try:
    import pytss.interface
except ImportError:
    # installing - there is no cffi yet
    ext_modules = []
else:
    # building bdist - cffi is here!
    ext_modules = [pytss.interface.ffi.verifier.get_extension()]


setup(
    name=__about__['__title__'],
    version=__about__['__version__'],

    description=__about__['__summary__'],
    license=open('LICENSE').read(),

    author=__about__['__author__'],
    author_email=__about__['__email__'],

    install_requires=[
        'cffi',
    ],
    extras_require={
        'tests': [
            'pep8',
            'pylint',
            'pytest',
        ],
    },
    tests_require=[
        'pytest',
    ],

    packages=[
        'pytss',
    ],

    package_data={
        'pytss': [
            'interface.h'
        ]
    },

    ext_modules=ext_modules,

    zip_safe=False,
)
